/** @packageDocumentation
 * @module iModelHubClient
 */
import { FileHandler } from "@bentley/itwin-client";
import { HttpRequestOptionsTransformer, IModelBaseHandler } from "./imodelhub/BaseHandler";
import { BriefcaseHandler } from "./imodelhub/Briefcases";
import { ChangeSetHandler } from "./imodelhub/ChangeSets";
import { CheckpointHandler } from "./imodelhub/Checkpoints";
import { CodeHandler } from "./imodelhub/Codes";
import { CustomRequestOptions } from "./imodelhub/CustomRequestOptions";
import { EventHandler } from "./imodelhub/Events";
import { GlobalEventHandler } from "./imodelhub/GlobalEvents";
import { IModelHandler, IModelsHandler } from "./imodelhub/iModels";
import { LockHandler } from "./imodelhub/Locks";
import { ThumbnailHandler } from "./imodelhub/Thumbnails";
import { UserInfoHandler } from "./imodelhub/Users";
import { VersionHandler } from "./imodelhub/Versions";
import { PermissionHandler } from "./imodelhub/Permissions";
import { CheckpointV2Handler } from "./imodelhub/CheckpointsV2";
/**
 * Base class that allows access to different iModel related Class handlers. Handlers should be accessed through an instance of this class, rather than constructed directly.
 * @public
 */
export declare abstract class IModelClient {
    protected _handler: IModelBaseHandler;
    private _fileHandler?;
    /**
     * Creates an instance of [[IModelClient]].
     * @param fileHandler File handler to handle file upload/download and file system operations.
     */
    constructor(baseHandler: IModelBaseHandler, fileHandler?: FileHandler, applicationVersion?: string);
    /**
     * Sets file handler for file upload/download.
     * @param fileHandler File handler to handle file upload/download and file system operations.
     */
    setFileHandler(fileHandler: FileHandler): void;
    /**
     * Gets file handler for file upload/download.
     * @returns File handler to handle file upload/download and file system operations.
     */
    get fileHandler(): FileHandler | undefined;
    /**
     * Get the handler for [[HubIModel]] instances.
     * @note Use [[IModelHubClient.IModel]] for the preferred single iModel per [[Project]] workflow.
     */
    get iModels(): IModelsHandler;
    /**
     * Get the handler for [[HubIModel]].
     * @beta
     */
    get iModel(): IModelHandler;
    /**
     * Get the handler for [[Briefcase]]s.
     * @internal
     */
    get briefcases(): BriefcaseHandler;
    /**
     * Get the handler for [[ChangeSet]]s.
     */
    get changeSets(): ChangeSetHandler;
    /**
     * Get the handler for [[Checkpoint]]s.
     * @internal
     */
    get checkpoints(): CheckpointHandler;
    /**
     * Get the handler for [[CheckpointV2]]s.
     * @internal
     */
    get checkpointsV2(): CheckpointV2Handler;
    /**
     * Get the handler for [[Lock]]s.
     * @internal
     */
    get locks(): LockHandler;
    /**
     * Get the handler for [Code]($common)s.
     * @internal
     */
    get codes(): CodeHandler;
    /**
     * Get the handler for [[UserInfo]].
     */
    get users(): UserInfoHandler;
    /**
     * Get the handler for [[Version]]s.
     */
    get versions(): VersionHandler;
    /**
     * Get the handler for [[Thumbnail]]s.
     */
    get thumbnails(): ThumbnailHandler;
    /**
     * Get the handler for [[IModelHubEvent]]s.
     */
    get events(): EventHandler;
    /**
     * Get the handler for [[IModelHubGlobalEvent]]s.
     * @internal
     */
    get globalEvents(): GlobalEventHandler;
    /**
     * Get the handler for permissions.
     * @internal
     */
    get permissions(): PermissionHandler | undefined;
    /**
     * Get the [CustomRequestOptions]($clients) object for controlling future request options.
     * @internal
     */
    get requestOptions(): CustomRequestOptions;
    /**
     * Adds a method that will be called for every request to modify HttpRequestOptions.
     * @param func Method that will be used to modify HttpRequestOptions.
     * @beta
     */
    use(transformer: HttpRequestOptionsTransformer): void;
}
//# sourceMappingURL=IModelClient.d.ts.map