"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IModelClient = void 0;
const BaseHandler_1 = require("./imodelhub/BaseHandler");
const Briefcases_1 = require("./imodelhub/Briefcases");
const ChangeSets_1 = require("./imodelhub/ChangeSets");
const Checkpoints_1 = require("./imodelhub/Checkpoints");
const Codes_1 = require("./imodelhub/Codes");
const Events_1 = require("./imodelhub/Events");
const GlobalEvents_1 = require("./imodelhub/GlobalEvents");
const iModels_1 = require("./imodelhub/iModels");
const Locks_1 = require("./imodelhub/Locks");
const Thumbnails_1 = require("./imodelhub/Thumbnails");
const Users_1 = require("./imodelhub/Users");
const Versions_1 = require("./imodelhub/Versions");
const CheckpointsV2_1 = require("./imodelhub/CheckpointsV2");
/**
 * Base class that allows access to different iModel related Class handlers. Handlers should be accessed through an instance of this class, rather than constructed directly.
 * @public
 */
class IModelClient {
    /**
     * Creates an instance of [[IModelClient]].
     * @param fileHandler File handler to handle file upload/download and file system operations.
     */
    constructor(baseHandler, fileHandler, applicationVersion) {
        this._handler = baseHandler;
        if (applicationVersion)
            this.use(BaseHandler_1.addApplicationVersion(applicationVersion));
        this._fileHandler = fileHandler || this._handler.getFileHandler();
        if (this._fileHandler)
            this._fileHandler.agent = this._handler.getAgent();
    }
    /**
     * Sets file handler for file upload/download.
     * @param fileHandler File handler to handle file upload/download and file system operations.
     */
    setFileHandler(fileHandler) {
        this._fileHandler = fileHandler;
        this._fileHandler.agent = this._handler.getAgent();
    }
    /**
     * Gets file handler for file upload/download.
     * @returns File handler to handle file upload/download and file system operations.
     */
    get fileHandler() {
        return this._fileHandler;
    }
    /**
     * Get the handler for [[HubIModel]] instances.
     * @note Use [[IModelHubClient.IModel]] for the preferred single iModel per [[Project]] workflow.
     */
    get iModels() {
        return new iModels_1.IModelsHandler(this._handler, this._fileHandler);
    }
    /**
     * Get the handler for [[HubIModel]].
     * @beta
     */
    get iModel() {
        return new iModels_1.IModelHandler(new iModels_1.IModelsHandler(this._handler, this._fileHandler));
    }
    /**
     * Get the handler for [[Briefcase]]s.
     * @internal
     */
    get briefcases() {
        return new Briefcases_1.BriefcaseHandler(this._handler, this, this._fileHandler);
    }
    /**
     * Get the handler for [[ChangeSet]]s.
     */
    get changeSets() {
        return new ChangeSets_1.ChangeSetHandler(this._handler, this._fileHandler);
    }
    /**
     * Get the handler for [[Checkpoint]]s.
     * @internal
     */
    get checkpoints() {
        return new Checkpoints_1.CheckpointHandler(this._handler, this._fileHandler);
    }
    /**
     * Get the handler for [[CheckpointV2]]s.
     * @internal
     */
    get checkpointsV2() {
        return new CheckpointsV2_1.CheckpointV2Handler(this._handler);
    }
    /**
     * Get the handler for [[Lock]]s.
     * @internal
     */
    get locks() {
        return new Locks_1.LockHandler(this._handler);
    }
    /**
     * Get the handler for [Code]($common)s.
     * @internal
     */
    get codes() {
        return new Codes_1.CodeHandler(this._handler);
    }
    /**
     * Get the handler for [[UserInfo]].
     */
    get users() {
        return new Users_1.UserInfoHandler(this._handler);
    }
    /**
     * Get the handler for [[Version]]s.
     */
    get versions() {
        return new Versions_1.VersionHandler(this._handler);
    }
    /**
     * Get the handler for [[Thumbnail]]s.
     */
    get thumbnails() {
        return new Thumbnails_1.ThumbnailHandler(this._handler);
    }
    /**
     * Get the handler for [[IModelHubEvent]]s.
     */
    get events() {
        return new Events_1.EventHandler(this._handler);
    }
    /**
     * Get the handler for [[IModelHubGlobalEvent]]s.
     * @internal
     */
    get globalEvents() {
        return new GlobalEvents_1.GlobalEventHandler(this._handler);
    }
    /**
     * Get the handler for permissions.
     * @internal
     */
    get permissions() {
        return undefined;
    }
    /**
     * Get the [CustomRequestOptions]($clients) object for controlling future request options.
     * @internal
     */
    get requestOptions() {
        return this._handler.getCustomRequestOptions();
    }
    /**
     * Adds a method that will be called for every request to modify HttpRequestOptions.
     * @param func Method that will be used to modify HttpRequestOptions.
     * @beta
     */
    use(transformer) {
        this._handler.use(transformer);
    }
}
exports.IModelClient = IModelClient;
//# sourceMappingURL=IModelClient.js.map