"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IModelBankFileSystemContextClient = void 0;
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module iModelBankClient
 */
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const context_registry_client_1 = require("@bentley/context-registry-client");
const itwin_client_1 = require("@bentley/itwin-client");
const Errors_1 = require("../imodelhub/Errors");
const IModelHubClientLoggerCategories_1 = require("../IModelHubClientLoggerCategories");
const loggerCategory = IModelHubClientLoggerCategories_1.IModelHubClientLoggerCategory.IModelBank;
/** @internal */
class IModelBankFileSystemContextClient {
    constructor(baseUri) {
        this.baseUri = baseUri;
    }
    async queryContextProps(requestContext, projectName) {
        requestContext.enter();
        const url = `${this.baseUri}/sv1.0/Repositories/Global--main/GlobalScope/Context`;
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, `Sending GET request to ${url}`);
        const queryOptions = {
            $select: "*",
            $filter: `name+eq+'${projectName}'`,
        };
        const options = {
            method: "GET",
            headers: { authorization: requestContext.accessToken.toTokenString() },
            qs: queryOptions,
            accept: "application/json",
        };
        const res = await itwin_client_1.request(requestContext, url, options);
        requestContext.enter();
        if (!res.body || !res.body.instances) {
            throw new Error(`Query to URL ${url} executed successfully, but did NOT return anything.`);
        }
        const props = res.body.instances;
        if (props.length === 0)
            throw new itwin_client_1.WsgError(bentleyjs_core_1.WSStatus.InstanceNotFound);
        if (props.length !== 1)
            throw new Errors_1.IModelHubClientError(bentleyjs_core_1.IModelHubStatus.InvalidArgumentError);
        bentleyjs_core_1.Logger.logTrace(loggerCategory, `Successful GET request to ${url}`);
        return props.map((value) => value.properties);
    }
    async queryAssetByName(requestContext, assetName) {
        const props = await this.queryContextProps(requestContext, assetName);
        requestContext.enter();
        const asset = new context_registry_client_1.Asset();
        asset.wsgId = asset.ecId = props[0].id;
        asset.name = props[0].name;
        return asset;
    }
    async queryProjectByName(requestContext, projectName) {
        const props = await this.queryContextProps(requestContext, projectName);
        requestContext.enter();
        const project = new context_registry_client_1.Project();
        project.wsgId = project.ecId = props[0].id;
        project.name = props[0].name;
        return project;
    }
    async createContext(requestContext, name) {
        requestContext.enter();
        const url = `${this.baseUri}/sv1.0/Repositories/Global--main/GlobalScope/Context`;
        bentleyjs_core_1.Logger.logInfo(loggerCategory, `Sending POST request to ${url}`);
        const body = { instance: { className: "Context", schemaName: "GlobalScope", properties: { name, id: "", description: "" } } };
        const options = {
            method: "POST",
            headers: { authorization: requestContext.accessToken.toTokenString() },
            body,
        };
        await itwin_client_1.request(requestContext, url, options);
    }
    async deleteContext(requestContext, contextId) {
        requestContext.enter();
        const url = `${this.baseUri}/sv1.0/Repositories/Global--main/GlobalScope/Context/${contextId}`;
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, `Sending DELETE request to ${url}`);
        const options = {
            method: "DELETE",
            headers: { authorization: requestContext.accessToken.toTokenString() },
        };
        await itwin_client_1.request(requestContext, url, options);
    }
}
exports.IModelBankFileSystemContextClient = IModelBankFileSystemContextClient;
//# sourceMappingURL=IModelBankFileSystemContextClient.js.map