"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IModelBaseHandler = exports.addCsrfHeader = exports.addApplicationVersion = exports.addHeader = void 0;
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module iModelHubClient
 */
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const itwin_client_1 = require("@bentley/itwin-client");
const CustomRequestOptions_1 = require("./CustomRequestOptions");
const Errors_1 = require("./Errors");
const applicationVersionHeaderName = "Application-Version";
/**
 * Provides default options for iModelHub requests.
 */
class DefaultIModelHubRequestOptionsProvider extends itwin_client_1.DefaultWsgRequestOptionsProvider {
    constructor(agent) {
        super();
        this._defaultOptions.errorCallback = Errors_1.IModelHubError.parse;
        this._defaultOptions.retryCallback = Errors_1.IModelHubError.shouldRetry;
        this._defaultOptions.agent = agent;
    }
}
/**
 * This function when used on IModelClient adds specified header to every request.
 * @beta
 */
function addHeader(name, valueFactory) {
    return (options) => {
        if (!options.headers)
            options.headers = {};
        options.headers[name] = valueFactory();
    };
}
exports.addHeader = addHeader;
/**
 * This function when used on IModelClient adds specified application version header to every request.
 * @beta
 */
function addApplicationVersion(version) {
    return addHeader(applicationVersionHeaderName, () => version);
}
exports.addApplicationVersion = addApplicationVersion;
/**
 * This function when used on IModelClient adds CSRF header to every request.
 * @beta
 */
function addCsrfHeader(headerName = "X-XSRF-TOKEN", cookieName = "XSRF-TOKEN") {
    return addHeader(headerName, () => {
        return document.cookie.split("; ").find((r) => r.startsWith(`${cookieName}=`)).split("=")[1];
    });
}
exports.addCsrfHeader = addCsrfHeader;
/**
 * This class acts as the WsgClient for other iModelHub Handlers.
 * @public
 */
class IModelBaseHandler extends itwin_client_1.WsgClient {
    /**
     * Create an instance of IModelBaseHandler.
     * @internal
     */
    constructor(keepAliveDuration = 30000, fileHandler) {
        super("sv1.1");
        this._customRequestOptions = new CustomRequestOptions_1.CustomRequestOptions();
        this._httpRequestOptionsTransformers = [];
        this.baseUrl = "https://api.bentley.com/imodelhub";
        this._fileHandler = fileHandler;
        const agentOptions = { keepAlive: keepAliveDuration > 0, keepAliveMsecs: keepAliveDuration, secureProtocol: "TLSv1_2_method" };
        if (itwin_client_1.RequestGlobalOptions.httpsProxy) {
            this._agent = itwin_client_1.RequestGlobalOptions.createHttpsProxy(agentOptions);
        }
        else {
            // eslint-disable-next-line @typescript-eslint/no-var-requires
            this._agent = require("https").Agent(agentOptions);
        }
    }
    /** @internal */
    getUrlSearchKey() { bentleyjs_core_1.assert(false, "Bentley cloud-specific method should be factored out of WsgClient base class"); return ""; }
    /**
     * @internal
     */
    formatContextIdForUrl(contextId) { return contextId; }
    /**
     * @internal
     */
    getFileHandler() { return this._fileHandler; }
    /**
     * Augment request options with defaults returned by the DefaultIModelHubRequestOptionsProvider. Note that the options passed in by clients override any defaults where necessary.
     * @param options Options the caller wants to augment with the defaults.
     * @returns Promise resolves after the defaults are setup.
     * @internal
     */
    async setupOptionDefaults(options) {
        if (!this._defaultIModelHubOptionsProvider)
            this._defaultIModelHubOptionsProvider = new DefaultIModelHubRequestOptionsProvider(this._agent);
        return this._defaultIModelHubOptionsProvider.assignOptions(options);
    }
    /**
     * Populates HTTP request options with additional data.
     * @param options Options that need to be populated.
     * @returns Options populated with additional data.
     * @internal
     */
    setupHttpOptions(options) {
        const httpOptions = Object.assign({}, options);
        for (const transformer of this._httpRequestOptionsTransformers) {
            transformer(httpOptions);
        }
        return httpOptions;
    }
    /**
     * Adds a method that will be called for every request to modify HttpRequestOptions.
     * @param func Method that will be used to modify HttpRequestOptions.
     * @beta
     */
    use(func) {
        this._httpRequestOptionsTransformers.push(func);
    }
    /**
     * Gets theRelyingPartyUrl for the service.
     * @returns RelyingPartyUrl for the service.
     * @internal
     */
    getRelyingPartyUrl() {
        /* eslint-disable deprecation/deprecation */
        if (bentleyjs_core_1.Config.App.has(IModelBaseHandler.configRelyingPartyUri))
            return `${bentleyjs_core_1.Config.App.get(IModelBaseHandler.configRelyingPartyUri)}/`;
        if (bentleyjs_core_1.Config.App.getBoolean(itwin_client_1.WsgClient.configUseHostRelyingPartyUriAsFallback, true)) {
            if (bentleyjs_core_1.Config.App.has(itwin_client_1.WsgClient.configHostRelyingPartyUri))
                return `${bentleyjs_core_1.Config.App.get(itwin_client_1.WsgClient.configHostRelyingPartyUri)}/`;
        }
        /* eslint-enable deprecation/deprecation */
        throw new Error(`RelyingPartyUrl not set. Set it in Config.App using key ${IModelBaseHandler.configRelyingPartyUri}`);
    }
    /**
     * Get the agent used for imodelhub connection pooling.
     * @returns The agent used for imodelhub connection pooling.
     * @internal
     */
    getAgent() {
        return this._agent;
    }
    /**
     * Get the URL of the service. This method attempts to discover and cache the URL from the URL Discovery Service. If not found uses the default URL provided by client implementations. Note that for consistency sake, the URL is stripped of any trailing "/"
     * @returns URL for the service
     * @internal
     */
    async getUrl(requestContext) {
        return super.getUrl(requestContext);
    }
    /**
     * Send a delete request. Sends a request without body.
     * @param requestContext The client request context
     * @param relativeUrlPath Relative path to the REST resource.
     * @param httpRequestOptions Additional options for the HTTP request.
     * @returns Promise resolves after successfully deleting REST resource at the specified path.
     */
    async delete(requestContext, relativeUrlPath, httpRequestOptions) {
        return super.delete(requestContext, relativeUrlPath, this.setupHttpOptions(httpRequestOptions));
    }
    /**
     * Delete a strongly typed instance. Sends a request body with a WSG instance.
     * @param requestContext The client request context
     * @param relativeUrlPath Relative path to the REST resource.
     * @param instance Instance to be deleted.
     * @param requestOptions WSG options for the request.
     * @returns Promise resolves after successfully deleting instance.
     */
    async deleteInstance(requestContext, relativeUrlPath, instance, requestOptions) {
        if (this._customRequestOptions.isSet) {
            if (!requestOptions) {
                requestOptions = {};
            }
            requestOptions.CustomOptions = this._customRequestOptions.insertCustomOptions(requestOptions.CustomOptions);
        }
        return super.deleteInstance(requestContext, relativeUrlPath, instance, requestOptions);
    }
    /**
     * Post a strongly typed instance. Sends a request body with a WSG instance.
     * @param requestContext The client request context
     * @param typedConstructor Used to construct the resulting instances from the response.
     * @param relativeUrlPath Relative path to the REST resource.
     * @param instance Strongly typed instance to be posted.
     * @param requestOptions WSG options for the request.
     * @param httpRequestOptions Additional options for the HTTP request.
     * @returns The posted instance that's returned back from the server.
     */
    async postInstance(requestContext, typedConstructor, relativeUrlPath, instance, requestOptions, httpRequestOptions) {
        if (this._customRequestOptions.isSet) {
            if (!requestOptions) {
                requestOptions = {};
            }
            requestOptions.CustomOptions = this._customRequestOptions.insertCustomOptions(requestOptions.CustomOptions);
        }
        return super.postInstance(requestContext, typedConstructor, relativeUrlPath, instance, requestOptions, this.setupHttpOptions(httpRequestOptions));
    }
    /**
     * Post multiple strongly typed instances. Sends a request body with WSG instances.
     * @param requestContext The client request context
     * @param typedConstructor Used to construct the resulting instances from the response.
     * @param relativeUrlPath Relative path to the REST resource.
     * @param instances Strongly typed instances to be posted.
     * @param requestOptions WSG options for the request.
     * @param httpRequestOptions Additional options for the HTTP request.
     * @returns The posted instances that's returned back from the server.
     */
    async postInstances(requestContext, typedConstructor, relativeUrlPath, instances, requestOptions, httpRequestOptions) {
        return super.postInstances(requestContext, typedConstructor, relativeUrlPath, instances, requestOptions, this.setupHttpOptions(httpRequestOptions));
    }
    /**
     * Get multiple strongly typed instances.
     * @param requestContext The client request context
     * @param typedConstructor Used to construct the resulting instances from the response.
     * @param relativeUrlPath Relative path to the REST resource.
     * @param queryOptions Query options.
     * @param httpRequestOptions Additional options for the HTTP request.
     * @returns Array of strongly typed instances.
     */
    async getInstances(requestContext, typedConstructor, relativeUrlPath, queryOptions, httpRequestOptions) {
        return super.getInstances(requestContext, typedConstructor, relativeUrlPath, queryOptions, this.setupHttpOptions(httpRequestOptions));
    }
    /**
     * Get a chunk of strongly typed instances.
     * @param requestContext Client request context
     * @param url Full path to the REST resource.
     * @param chunkedQueryContext Chunked query context
     * @param typedConstructor Constructor function for the type
     * @param queryOptions Query options.
     * @param httpRequestOptions Additional options for the HTTP request.
     * @returns Array of strongly typed instances.
     */
    async getInstancesChunk(requestContext, url, chunkedQueryContext, typedConstructor, queryOptions, httpRequestOptions) {
        return super.getInstancesChunk(requestContext, url, chunkedQueryContext, typedConstructor, queryOptions, this.setupHttpOptions(httpRequestOptions));
    }
    /**
     * Get multiple strongly typed instances. Sends query in the request's body. This can be used for queries that are too long to fit in URL.
     * @param requestContext The client request context
     * @param typedConstructor Used to construct the resulting instances from the response.
     * @param relativeUrlPath Relative path to the REST resource.
     * @param queryOptions Query options.
     * @param httpRequestOptions Additional options for the HTTP request.
     * @returns Array of strongly typed instances.
     */
    async postQuery(requestContext, typedConstructor, relativeUrlPath, queryOptions, httpRequestOptions) {
        return super.postQuery(requestContext, typedConstructor, relativeUrlPath, queryOptions, this.setupHttpOptions(httpRequestOptions));
    }
    /**
     * Get an instance of CustomRequestOptions, which can be used to set custom request parameters for all future requests made by this handler.
     * @internal
     */
    getCustomRequestOptions() {
        return this._customRequestOptions;
    }
}
exports.IModelBaseHandler = IModelBaseHandler;
IModelBaseHandler.configRelyingPartyUri = "imjs_imodelhub_relying_party_uri";
//# sourceMappingURL=BaseHandler.js.map