"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module iModelHubClient
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChangeSetHandler = exports.ChangeSetQuery = exports.ChangeSet = exports.ChangesType = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const itwin_client_1 = require("@bentley/itwin-client");
const IModelHubClientLoggerCategories_1 = require("../IModelHubClientLoggerCategories");
const Errors_1 = require("./Errors");
const HubQuery_1 = require("./HubQuery");
const loggerCategory = IModelHubClientLoggerCategories_1.IModelHubClientLoggerCategory.IModelHub;
/**
 * Specifies types of changes in a [[ChangeSet]].
 * @public
 */
var ChangesType;
(function (ChangesType) {
    /** [[ChangeSet]] contains regular file changes (e.g. changes to elements or models). */
    ChangesType[ChangesType["Regular"] = 0] = "Regular";
    /** [[ChangeSet]] only contains schema changes. */
    ChangesType[ChangesType["Schema"] = 1] = "Schema";
    /** [[ChangeSet]] contains definition changes. */
    ChangesType[ChangesType["Definition"] = 2] = "Definition";
    /** [[ChangeSet]] contains spatial data changes. */
    ChangesType[ChangesType["SpatialData"] = 4] = "SpatialData";
    /** [[ChangeSet]] contains sheets and drawings changes. */
    ChangesType[ChangesType["SheetsAndDrawings"] = 8] = "SheetsAndDrawings";
    /** [[ChangeSet]] contains views and model changes. */
    ChangesType[ChangesType["ViewsAndModels"] = 16] = "ViewsAndModels";
    /** [[ChangeSet]] contains changes of global properties. */
    ChangesType[ChangesType["GlobalProperties"] = 32] = "GlobalProperties";
})(ChangesType = exports.ChangesType || (exports.ChangesType = {}));
/**
 * [ChangeSet]($docs/learning/Glossary.md#changeset) represents a file containing changes to the iModel. A single ChangeSet contains changes made on a single [[Briefcase]] file and pushed as a single file. ChangeSets form a linear change history of the iModel. If a user wants to push their changes to iModelHub, they first have to merge all ChangeSet they do not have yet. Only a single briefcase is allowed to push their changes at a time.
 * @public
 */
let ChangeSet = class ChangeSet extends itwin_client_1.WsgInstance {
    /** Parsed number of this ChangeSet file size. */
    get fileSizeNumber() {
        if (this._fileSizeNumber)
            return this._fileSizeNumber;
        this._fileSizeNumber = parseInt(this.fileSize, 10);
        return this._fileSizeNumber;
    }
};
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Id")
], ChangeSet.prototype, "id", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.FileName")
], ChangeSet.prototype, "fileName", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Description")
], ChangeSet.prototype, "description", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.FileSize")
], ChangeSet.prototype, "fileSize", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Index")
], ChangeSet.prototype, "index", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.ParentId")
], ChangeSet.prototype, "parentId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.SeedFileId")
], ChangeSet.prototype, "seedFileId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.BriefcaseId")
], ChangeSet.prototype, "briefcaseId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.UserCreated")
], ChangeSet.prototype, "userCreated", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.PushDate")
], ChangeSet.prototype, "pushDate", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.ContainingChanges")
], ChangeSet.prototype, "changesType", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.IsUploaded")
], ChangeSet.prototype, "isUploaded", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[FileAccessKey].relatedInstance[AccessKey].properties.DownloadUrl")
], ChangeSet.prototype, "downloadUrl", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[FileAccessKey].relatedInstance[AccessKey].properties.UploadUrl")
], ChangeSet.prototype, "uploadUrl", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[CreatedByApplication].relatedInstance[Application].properties.Id")
], ChangeSet.prototype, "applicationId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[CreatedByApplication].relatedInstance[Application].properties.Name")
], ChangeSet.prototype, "applicationName", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[HasBridgeProperties](direction:forward).relatedInstance[BridgeProperties].properties.JobId")
], ChangeSet.prototype, "bridgeJobId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[HasBridgeProperties](direction:forward).relatedInstance[BridgeProperties].properties.Users")
], ChangeSet.prototype, "bridgeUsers", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[HasBridgeProperties](direction:forward).relatedInstance[BridgeProperties].properties.ChangedFiles")
], ChangeSet.prototype, "bridgeChangedFiles", void 0);
ChangeSet = __decorate([
    itwin_client_1.ECJsonTypeMap.classToJson("wsg", "iModelScope.ChangeSet", { schemaPropertyName: "schemaName", classPropertyName: "className" })
], ChangeSet);
exports.ChangeSet = ChangeSet;
/**
 * Query object for getting [[ChangeSet]]s. You can use this to modify the query. See [[ChangeSetHandler.get]].
 * @public
 */
class ChangeSetQuery extends HubQuery_1.StringIdQuery {
    /** Constructor that sets default page size. */
    constructor() {
        super();
        this.pageSize(ChangeSetQuery.defaultPageSize);
    }
    /**
     * Clone current query.
     * @returns Cloned query.
     */
    clone() {
        const changeSetQuery = new ChangeSetQuery();
        changeSetQuery._query = Object.assign({}, this.getQueryOptions());
        changeSetQuery._byId = this.getId();
        return changeSetQuery;
    }
    /**
     * Query will additionally select [[ChangeSet]] file download URL. This is needed to use the ChangeSet object with [[ChangeSetHandler.download]].
     * @returns This query.
     */
    selectDownloadUrl() {
        HubQuery_1.addSelectFileAccessKey(this._query);
        return this;
    }
    /**
     * Query will additionally select data about application that created this [[ChangeSet]].
     * @returns This query.
     */
    selectApplicationData() {
        HubQuery_1.addSelectApplicationData(this._query);
        return this;
    }
    /** @internal */
    checkValue(id) {
        Errors_1.ArgumentCheck.validChangeSetId("id", id);
    }
    /**
     * Query [[ChangeSet]]s that are after the specified ChangeSet. This overrides any previously applied ChangeSetQuery filters. Query will return all of the ChangeSets that are newer than the one specified by id. ChangeSet specified by the id will not be included in the results. Returned ChangeSets will be in an ascending order.
     * @param id Id of a ChangeSet.
     * @returns This query.
     */
    fromId(id) {
        Errors_1.ArgumentCheck.validChangeSetId("id", id);
        this._query.$filter = `FollowingChangeSet-backward-ChangeSet.Id+eq+'${id}'`;
        return this;
    }
    /**
     * Change the order of results to be from newest [[ChangeSet]]s to the oldest ones.
     * @returns This query.
     */
    latest() {
        this._query.$orderby = "Index+desc";
        return this;
    }
    /**
     * Query [[ChangeSet]]s between two specified ChangeSets. This overrides any previously applied ChangeSetQuery filters. This query will work when either of the ChangeSet ids points to an earlier ChangeSet. Latest ChangeSet specified by this range will be included in the results, but the earliest will be excluded. If the second ChangeSet id is not specified, it's assumed that it's the same as an empty id, and query will return all ChangeSets from the start up to the ChangeSet with the first id. Returned ChangeSets will be in an ascending order.
     * @param firstChangeSetId Id of the first changeSet.
     * @param secondChangeSetId Id of the second changeSet.
     * @returns This query.
     */
    betweenChangeSets(firstChangeSetId, secondChangeSetId) {
        Errors_1.ArgumentCheck.validChangeSetId("firstChangeSetId", firstChangeSetId);
        if (secondChangeSetId)
            Errors_1.ArgumentCheck.validChangeSetId("secondChangeSetId", secondChangeSetId);
        let query;
        if (!secondChangeSetId) {
            query = `CumulativeChangeSet-backward-ChangeSet.Id+eq+'${firstChangeSetId}'`;
        }
        else {
            query = `(CumulativeChangeSet-backward-ChangeSet.Id+eq+'${firstChangeSetId}'`;
            query += `+and+FollowingChangeSet-backward-ChangeSet.Id+eq+'${secondChangeSetId}')`;
            query += `+or+(CumulativeChangeSet-backward-ChangeSet.Id+eq+'${secondChangeSetId}'`;
            query += `+and+FollowingChangeSet-backward-ChangeSet.Id+eq+'${firstChangeSetId}')`;
        }
        this._query.$filter = query;
        return this;
    }
    /**
     * Query [[ChangeSet]]s included in the specified [[Version]]. This overrides any previously applied ChangeSetQuery filters. Query will return all of the ChangeSets from the start up to the one specified by versionId. ChangeSet specified by versionId will be included in the results. Returned ChangeSets will be in an ascending order.
     * @param versionId Id of the version.
     * @returns This query.
     */
    getVersionChangeSets(versionId) {
        Errors_1.ArgumentCheck.validGuid("versionId", versionId);
        this._query.$filter = `CumulativeChangeSet-backward-Version.Id+eq+'${versionId}'`;
        return this;
    }
    /**
     * Query [[ChangeSet]]s after the specified [[Version]]. This overrides any previously applied ChangeSetQuery filters. Query will return all of the ChangeSets that are newer than the one specified by versionId. ChangeSet specified by versionId will not be included in the results. Returned ChangeSets will be in an ascending order.
     * @param versionId Id of the version.
     * @returns This query.
     */
    afterVersion(versionId) {
        Errors_1.ArgumentCheck.validGuid("versionId", versionId);
        this._query.$filter = `FollowingChangeSet-backward-Version.Id+eq+'${versionId}'`;
        return this;
    }
    /**
     * Query [[ChangeSet]]s between two specified [[Version]]s. This overrides any previously applied ChangeSetQuery filters. This query will work when either of the Version ids points to an earlier ChangeSet. Latest ChangeSet specified by this range will be included in the results, but the earliest will be excluded. Returned ChangeSets will be in an ascending order.
     * @param sourceVersionId Id of the source version.
     * @param destinationVersionId Id of the destination version.
     * @returns This query.
     */
    betweenVersions(sourceVersionId, destinationVersionId) {
        Errors_1.ArgumentCheck.validGuid("sourceVersionId", sourceVersionId);
        Errors_1.ArgumentCheck.validGuid("destinationVersionId", destinationVersionId);
        let query;
        query = `(FollowingChangeSet-backward-Version.Id+eq+'${sourceVersionId}'`;
        query += `+and+CumulativeChangeSet-backward-Version.Id+eq+'${destinationVersionId}')`;
        query += `+or+(FollowingChangeSet-backward-Version.Id+eq+'${destinationVersionId}'`;
        query += `+and+CumulativeChangeSet-backward-Version.Id+eq+'${sourceVersionId}')`;
        this._query.$filter = query;
        return this;
    }
    /**
     * Query [[ChangeSet]]s between the specified [[Version]] and another [[ChangeSet]]. This overrides any previously applied ChangeSetQuery filters. This query will work when either versionId or changeSetId points to an earlier ChangeSet. Latest ChangeSet specified by this range will be included in the results, but the earliest will be excluded. Returned ChangeSets will be in an ascending order.
     * @param versionId Id of the version.
     * @param changeSetId Id of the changeSet.
     * @returns This query.
     */
    betweenVersionAndChangeSet(versionId, changeSetId) {
        Errors_1.ArgumentCheck.validGuid("versionId", versionId);
        Errors_1.ArgumentCheck.validChangeSetId("changeSetId", changeSetId);
        let query;
        query = `(CumulativeChangeSet-backward-Version.Id+eq+'${versionId}'+and+FollowingChangeSet-backward-ChangeSet.Id+eq+'${changeSetId}')`;
        query += `+or+`;
        query += `(FollowingChangeSet-backward-Version.Id+eq+'${versionId}'+and+CumulativeChangeSet-backward-ChangeSet.Id+eq+'${changeSetId}')`;
        this._query.$filter = query;
        return this;
    }
    /**
     * Query changeSets by the seed file id. Should be obsolete, because seed file replacement is deprecated.
     * @param seedFileId Id of the seed file.
     * @returns This query.
     * @internal
     */
    bySeedFileId(seedFileId) {
        Errors_1.ArgumentCheck.validGuid("seedFileId", seedFileId);
        this.addFilter(`SeedFileId+eq+'${seedFileId}'`);
        return this;
    }
    /** Select all bridge properties. */
    selectBridgeProperties() {
        if (!this._query.$select)
            this._query.$select = "*";
        return this.addSelect(`HasBridgeProperties-forward-BridgeProperties.*`);
    }
}
exports.ChangeSetQuery = ChangeSetQuery;
/**
 * Default page size which is used when querying ChangeSets
 * @internal
 */
ChangeSetQuery.defaultPageSize = 1000;
/** Queue for limiting number of promises executed in parallel. */
class ParallelQueue {
    constructor() {
        this._queue = [];
        this._parallelDownloads = 10;
    }
    /** Add a promise to the queue. */
    push(downloadFunc) {
        this._queue.push(downloadFunc);
    }
    /** Wait for all promises in the queue to finish. */
    async waitAll() {
        let i = 0;
        const promises = new Array();
        const indexes = new Array();
        const completed = new Array();
        while (this._queue.length > 0 || promises.length > 0) {
            while (this._queue.length > 0 && promises.length < this._parallelDownloads) {
                const currentIndex = i++;
                promises.push(this._queue[0]().then(() => completed.push(currentIndex)));
                indexes.push(currentIndex);
                this._queue.shift();
            }
            await Promise.race(promises);
            while (completed.length > 0) {
                const completedIndex = completed.shift();
                const index = indexes.findIndex((value) => value === completedIndex);
                if (index !== undefined) {
                    promises.splice(index, 1);
                    indexes.splice(index, 1);
                }
            }
        }
    }
}
/** Object for tracking changeSets download progress. */
class DownloadProgress {
    constructor(totalSize) {
        this._downloadedSize = 0;
        this._totalSize = totalSize;
    }
    /** Gets total changeSets size to download. */
    get totalSize() {
        return this._totalSize;
    }
    /** Gets currently downloaded changeSets size. */
    get downloadedSize() {
        return this._downloadedSize;
    }
    /** Sets currently downloaded changeSets size. */
    set downloadedSize(value) {
        this._downloadedSize = value;
    }
}
/**
 * Handler for managing [[ChangeSet]]s. Use [[IModelClient.ChangeSets]] to get an instance of this class. In most cases, you should use [BriefcaseDb]($backend) methods instead.
 * @public
 */
class ChangeSetHandler {
    /**
     * Constructor for ChangeSetHandler. Should use [[IModelClient]] instead of directly constructing this.
     * @param handler Handler for WSG requests.
     * @param fileHandler Handler for file system.
     * @internal
     */
    constructor(handler, fileHandler) {
        this._handler = handler;
        this._fileHandler = fileHandler;
    }
    /** Get relative url for ChangeSet requests.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param changeSetId Id of the ChangeSet.
     * @internal
     */
    getRelativeUrl(iModelId, changeSetId) {
        return `/Repositories/iModel--${iModelId}/iModelScope/ChangeSet/${changeSetId || ""}`;
    }
    /** Get the [[ChangeSet]]s for the iModel.
     * @param requestContext The client request context
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param query Optional query object to filter the queried ChangeSets or select different data from them.
     * @returns ChangeSets that match the query.
     * @throws [WsgError]($itwin-client) with [WSStatus.InstanceNotFound]($bentley) if [[InstanceIdQuery.byId]] is used and a [[ChangeSet]] with the specified id could not be found.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async get(requestContext, iModelId, query = new ChangeSetQuery()) {
        requestContext.enter();
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        bentleyjs_core_1.Logger.logTrace(loggerCategory, `Querying ChangeSets`, () => ({ iModelId }));
        const id = query.getId();
        const changeSets = await this._handler.getInstances(requestContext, ChangeSet, this.getRelativeUrl(iModelId, id), query.getQueryOptions());
        requestContext.enter();
        return changeSets;
    }
    /** Get the chunk of [[ChangeSet]]s for the iModel.
     * @param requestContext The client request context.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param url [[ChangeSet]]s query url.
     * @param chunkedQueryContext [[ChangeSet]]s chunked query context.
     * @param queryOptions Request query options.
     * @returns [[ChangeSet]]s chunk that match the query.
     * @throws [[WsgError]] with [WSStatus.InstanceNotFound]($bentley) if [[InstanceIdQuery.byId]] is used and a [[ChangeSet]] with the specified id could not be found.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async getChunk(requestContext, iModelId, url, chunkedQueryContext, queryOptions) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, `Querying ChangeSets chunk`, () => ({ iModelId }));
        const changeSets = await this._handler.getInstancesChunk(requestContext, url, chunkedQueryContext, ChangeSet, queryOptions);
        requestContext.enter();
        return changeSets;
    }
    /**
     * Download the [[ChangeSet]]s that match provided query. If you want to [pull]($docs/learning/Glossary.md#pull) and [merge]($docs/learning/Glossary.md#merge) ChangeSets from iModelHub to your [[Briefcase]], you should use [BriefcaseDb.pullAndMergeChanges]($backend) instead.
     *
     * This method creates the directory containing the ChangeSets if necessary. If there is an error in downloading some of the ChangeSets, all partially downloaded ChangeSets are deleted from disk.
     * @param requestContext The client request context
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param query Query that defines ChangeSets to download.
     * @param path Path of directory where the ChangeSets should be downloaded.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley), if one of the required arguments is undefined or empty.
     * @param progressCallback Callback for tracking progress.
     * @throws [ResponseError]($itwin-client) if the download fails.
     * @internal
     */
    async download(requestContext, iModelId, query, path, progressCallback) {
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        Errors_1.ArgumentCheck.defined("path", path);
        requestContext.enter();
        if (typeof window !== "undefined")
            throw Errors_1.IModelHubClientError.browser();
        if (!this._fileHandler)
            throw Errors_1.IModelHubClientError.fileHandler();
        const changeSetsToDownloadQuery = query.clone();
        changeSetsToDownloadQuery.select("FileSize");
        const changeSetsToDownload = await this.get(requestContext, iModelId, changeSetsToDownloadQuery);
        requestContext.enter();
        if (changeSetsToDownload.length === 0)
            return new Array();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, `Downloading ${changeSetsToDownload.length} changesets`);
        let totalSize = 0;
        changeSetsToDownload.forEach((value) => totalSize += value.fileSizeNumber);
        const downloadProgress = new DownloadProgress(totalSize);
        query.selectDownloadUrl();
        const url = await this._handler.getUrl(requestContext) + this.getRelativeUrl(iModelId, query.getId());
        requestContext.enter();
        const queryOptions = query.getQueryOptions();
        const chunkedQueryContext = queryOptions ? itwin_client_1.ChunkedQueryContext.create(queryOptions) : undefined;
        const changeSets = new Array();
        do {
            const changeSetsChunk = await this.getChunk(requestContext, iModelId, url, chunkedQueryContext, queryOptions);
            requestContext.enter();
            await this.downloadChunk(requestContext, iModelId, changeSetsChunk, path, downloadProgress, progressCallback);
            requestContext.enter();
            changeSets.push(...changeSetsChunk);
        } while (chunkedQueryContext && !chunkedQueryContext.isQueryFinished);
        bentleyjs_core_1.Logger.logInfo(loggerCategory, `Downloaded ${changeSets.length} changesets`);
        return changeSets;
    }
    /** Download the chunk of [[ChangeSet]]s for the iModel.
     * @param requestContext The client request context.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param changeSets [[ChangeSet]]s chunk to download.
     * @param path Path of directory where the ChangeSets should be downloaded.
     * @param downloadProgress Object that tracks [[ChangeSet]]s download progress.
     * @param progressCallback Callback for tracking progress.
     * @throws [ResponseError]($itwin-client) if the download fails.
     */
    async downloadChunk(requestContext, iModelId, changeSets, path, downloadProgress, progressCallback) {
        requestContext.enter();
        changeSets.forEach((changeSet) => {
            if (!changeSet.downloadUrl)
                throw Errors_1.IModelHubClientError.missingDownloadUrl("changeSets");
        });
        bentleyjs_core_1.Logger.logInfo(loggerCategory, `Downloading ${changeSets.length} changesets chunk`);
        // Sort changesets by file size ascending. This way we increase probability SAS token won't expire for small changesets
        const sortedChangeSets = Array.from(changeSets).sort((a, b) => a.fileSizeNumber < b.fileSizeNumber ? -1 : a.fileSizeNumber === b.fileSizeNumber ? 0 : 1);
        const queue = new ParallelQueue();
        const fileHandler = this._fileHandler;
        sortedChangeSets.forEach((changeSet) => queue.push(async () => {
            const downloadPath = fileHandler.join(path, changeSet.fileName);
            let previouslyDownloaded = 0;
            const callback = (progress) => {
                downloadProgress.downloadedSize += (progress.loaded - previouslyDownloaded);
                previouslyDownloaded = progress.loaded;
                progressCallback({ loaded: downloadProgress.downloadedSize, total: downloadProgress.totalSize, percent: downloadProgress.downloadedSize / downloadProgress.totalSize });
            };
            if (this.wasChangeSetDownloaded(downloadPath, changeSet.fileSizeNumber, changeSet)) {
                if (progressCallback)
                    callback({ percent: 100, total: changeSet.fileSizeNumber, loaded: changeSet.fileSizeNumber });
                return;
            }
            await this.downloadChangeSetWithRetry(requestContext, iModelId, changeSet, downloadPath, progressCallback ? callback : undefined);
            requestContext.enter();
        }));
        await queue.waitAll();
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, `Finished downloading changesets chunk`);
    }
    /** Download single [[ChangeSet]] with retry if SAS url expired or download failed.
     * @param requestContext The client request context.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param changeSet [[ChangeSet]] to download.
     * @param downloadPath Path where the ChangeSet should be downloaded.
     * @param changeSetProgress Callback for tracking progress.
     * @throws [ResponseError]($itwin-client) if the download fails.
     */
    async downloadChangeSetWithRetry(requestContext, iModelId, changeSet, downloadPath, changeSetProgress) {
        requestContext.enter();
        try {
            await this.downloadChangeSet(requestContext, changeSet, downloadPath, changeSetProgress);
            requestContext.enter();
            return;
        }
        catch (error) {
            requestContext.enter();
            if (!(error instanceof itwin_client_1.SasUrlExpired || error instanceof itwin_client_1.DownloadFailed))
                throw error;
            bentleyjs_core_1.Logger.logWarning(loggerCategory, `ChangeSet download failed, refreshing downloadUrl to retry download.`, () => ({ iModelId, id: changeSet.id }));
            const changeSetQuery = new ChangeSetQuery();
            changeSetQuery.byId(changeSet.id);
            changeSetQuery.selectDownloadUrl();
            const refreshedChangeSets = await this.get(requestContext, iModelId, changeSetQuery);
            requestContext.enter();
            if (refreshedChangeSets.length === 0)
                throw error;
            await this.downloadChangeSet(requestContext, refreshedChangeSets[0], downloadPath, changeSetProgress);
            requestContext.enter();
            return;
        }
    }
    /** Download single [[ChangeSet]].
     * @param requestContext The client request context.
     * @param changeSet [[ChangeSet]] to download.
     * @param downloadPath Path where the ChangeSet should be downloaded.
     * @param changeSetProgress Callback for tracking progress.
     * @throws [ResponseError]($itwin-client) if the download fails.
     */
    async downloadChangeSet(requestContext, changeSet, downloadPath, changeSetProgress) {
        requestContext.enter();
        try {
            await this._fileHandler.downloadFile(requestContext, changeSet.downloadUrl, downloadPath, changeSet.fileSizeNumber, changeSetProgress);
            requestContext.enter();
        }
        catch (error) {
            requestContext.enter();
            // Note: If the cache was shared across processes, it's possible that the download was completed by another process
            if (this.wasChangeSetDownloaded(downloadPath, changeSet.fileSizeNumber, changeSet))
                return;
            throw error;
        }
    }
    /** Checks if ChangeSet file was already downloaded.
     * @param path Path of file where the ChangeSet should be downloaded.
     * @param expectedFileSize Size of the file that's being downloaded.
     * @param changeSet ChangeSet that's being downloaded.
     */
    wasChangeSetDownloaded(path, expectedFileSize, changeSet) {
        if (!this._fileHandler)
            return false;
        if (!this._fileHandler.exists(path))
            return false;
        const actualFileSize = this._fileHandler.getFileSize(path);
        if (actualFileSize === expectedFileSize)
            return true;
        bentleyjs_core_1.Logger.logError(loggerCategory, `ChangeSet size ${actualFileSize} does not match the expected size ${expectedFileSize}. Deleting it so that it can be refetched`, () => (changeSet));
        try {
            this._fileHandler.unlink(path);
        }
        catch (error) {
            bentleyjs_core_1.Logger.logError(loggerCategory, `Cannot delete ChangeSet file at ${path}`);
        }
        return false;
    }
    /**
     * Upload a [[ChangeSet]] file. If you want to [push]($docs/learning/Glossary.md#push) your changes to iModelHub, use [BriefcaseDb.pushChanges]($backend) instead. This method is only a part of that workflow.
     *
     * ChangeSets have to be uploaded in a linear order. If another user is uploading, or changeSet.parentId does not point to the latest ChangeSet on iModelHub, this method will fail. User will have to download all of the newer ChangeSets, merge them into their [[Briefcase]] and calculate a new ChangeSet id.
     * @param requestContext The client request context
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param changeSet Information of the ChangeSet to be uploaded.
     * @param path Path of the ChangeSet file to be uploaded.
     * @param progressCallback Callback for tracking upload progress.
     * @throws [IModelHubStatus.BriefcaseDoesNotBelongToUser]($bentley) if Briefcase specified by changeSet.briefcaseId belongs to another user.
     * @throws [IModelHubStatus.AnotherUserPushing]($bentley) if another user is currently uploading a ChangeSet.
     * @throws [IModelHubStatus.PullIsRequired]($bentley) if there are newer ChangeSets on iModelHub, that need to be downloaded and merged, before upload is possible.
     * @throws [IModelHubStatus.ChangeSetAlreadyExists]($bentley) if a ChangeSet with this id already exists. This usually happens if previous upload attempt has succeeded.
     * @throws [IModelHubStatus.ChangeSetPointsToBadSeed]($bentley) if changeSet.seedFileId is not set to the correct file id. That file id should match to the value written to the Briefcase file.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     * @internal
     */
    async create(requestContext, iModelId, changeSet, path, progressCallback) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Started uploading ChangeSet", () => (Object.assign({ iModelId }, changeSet)));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        Errors_1.ArgumentCheck.defined("changeSet", changeSet);
        Errors_1.ArgumentCheck.defined("path", path);
        if (typeof window !== "undefined")
            throw Errors_1.IModelHubClientError.browser();
        if (!this._fileHandler)
            throw Errors_1.IModelHubClientError.fileHandler();
        if (!this._fileHandler.exists(path) || this._fileHandler.isDirectory(path))
            throw Errors_1.IModelHubClientError.fileNotFound();
        const postChangeSet = await this._handler.postInstance(requestContext, ChangeSet, this.getRelativeUrl(iModelId), changeSet);
        await this._fileHandler.uploadFile(requestContext, postChangeSet.uploadUrl, path, progressCallback);
        requestContext.enter();
        postChangeSet.uploadUrl = undefined;
        postChangeSet.downloadUrl = undefined;
        postChangeSet.isUploaded = true;
        const confirmChangeSet = await this._handler.postInstance(requestContext, ChangeSet, this.getRelativeUrl(iModelId, postChangeSet.id), postChangeSet);
        requestContext.enter();
        changeSet.isUploaded = true;
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Finished uploading ChangeSet", () => (Object.assign({ iModelId }, changeSet)));
        return confirmChangeSet;
    }
}
exports.ChangeSetHandler = ChangeSetHandler;
//# sourceMappingURL=ChangeSets.js.map