"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module iModelHubClient
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CheckpointHandler = exports.CheckpointQuery = exports.Checkpoint = void 0;
const urllib = require("url");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const itwin_client_1 = require("@bentley/itwin-client");
const IModelHubClientLoggerCategories_1 = require("../IModelHubClientLoggerCategories");
const Errors_1 = require("./Errors");
const HubQuery_1 = require("./HubQuery");
const loggerCategory = IModelHubClientLoggerCategories_1.IModelHubClientLoggerCategory.IModelHub;
/**
 * Checkpoint is a copy of the master file, that is intended to be read-only and reduces amount of merging required to get an iModel to a specific previous state.
 *
 * File properties describe the file that would be downloaded through downloadUrl.
 * @internal
 */
let Checkpoint = class Checkpoint extends itwin_client_1.WsgInstance {
};
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.FileName")
], Checkpoint.prototype, "fileName", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.FileDescription")
], Checkpoint.prototype, "fileDescription", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.FileSize")
], Checkpoint.prototype, "fileSize", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.FileId")
], Checkpoint.prototype, "fileId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.State")
], Checkpoint.prototype, "state", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.MergedChangeSetId")
], Checkpoint.prototype, "mergedChangeSetId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.CreatedDate")
], Checkpoint.prototype, "createdDate", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[FileAccessKey].relatedInstance[AccessKey].properties.DownloadUrl")
], Checkpoint.prototype, "downloadUrl", void 0);
Checkpoint = __decorate([
    itwin_client_1.ECJsonTypeMap.classToJson("wsg", "iModelScope.Checkpoint", { schemaPropertyName: "schemaName", classPropertyName: "className" })
], Checkpoint);
exports.Checkpoint = Checkpoint;
/**
 * Query object for getting [[Checkpoint]]s. You can use this to modify the [[CheckpointHandler.get]] results.
 * @internal
 */
class CheckpointQuery extends itwin_client_1.WsgQuery {
    /** Query will return closest [[Checkpoint]] to target [[ChangeSet]], based on ChangeSets size.
     * This query can return a Checkpoint that is ahead of the specified ChangeSet, if reversing ChangeSets would be faster than merging forward. This resets all previously set filters.
     * @returns This query.
     */
    nearestCheckpoint(targetChangeSetId) {
        this.filter(`NearestCheckpoint-backward-ChangeSet.Id+eq+'${targetChangeSetId}'`);
        return this;
    }
    /** Query will return closest [[Checkpoint]] to target [[ChangeSet]] that does not exceed the specified ChangeSet.
     * This query returns a closest Checkpoint that will reach target ChangeSet by only merging forward. This resets all previously set filters.
     * @returns This query.
     */
    precedingCheckpoint(targetChangeSetId) {
        this.filter(`PrecedingCheckpoint-backward-ChangeSet.Id+eq+'${targetChangeSetId}'`);
        return this;
    }
    /** Query will return [[Checkpoint]] with specified [[ChangeSet]] id.
     * @returns This query.
     */
    byChangeSetId(changeSetId) {
        this.addFilter(`MergedChangeSetId+eq+'${changeSetId}'`, "and");
        return this;
    }
    /** Query will additionally select [[Checkpoint]] file download URL. This is needed to use the Checkpoint object with [[CheckpointHandler.download]].
     * @returns This query.
     */
    selectDownloadUrl() {
        HubQuery_1.addSelectFileAccessKey(this._query);
        return this;
    }
}
exports.CheckpointQuery = CheckpointQuery;
/**
 * Handler for managing [[Checkpoint]]s. Use [[IModelClient.checkpoints]] to get an instance of this class.
 * In most cases, you should use [BriefcaseDb]($backend) methods instead.
 * @internal
 */
class CheckpointHandler {
    /** Constructor for CheckpointHandler. Use [[IModelClient]] instead of directly constructing this.
     * @param handler Handler for WSG requests.
     * @param fileHandler Handler for file system.
     * @internal
     */
    constructor(handler, fileHandler) {
        this._handler = handler;
        this._fileHandler = fileHandler;
    }
    /** Get relative url for Checkpoint requests.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @internal
     */
    getRelativeUrl(iModelId) {
        return `/Repositories/iModel--${iModelId}/iModelScope/Checkpoint/`;
    }
    /** Get the [[Checkpoint]]s.
     * @param requestContext The client request context
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param query Optional query object to filter the queried Checkpoints or select different data from them.
     * @returns Checkpoints that match the query.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async get(requestContext, iModelId, query = new CheckpointQuery()) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Querying checkpoints for iModel", () => ({ iModelId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        const checkpoints = await this._handler.getInstances(requestContext, Checkpoint, this.getRelativeUrl(iModelId), query.getQueryOptions());
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Queried checkpoints for iModel", () => ({ iModelId, count: checkpoints.length }));
        return checkpoints;
    }
    /**
     * Make url safe for logging by removing sensitive information
     * @param url input url that will be strip of search and query parameters and replace them by ... for security reason
     */
    static getSafeUrlForLogging(url) {
        const safeToLogDownloadUrl = urllib.parse(url);
        if (safeToLogDownloadUrl.search && safeToLogDownloadUrl.search.length > 0)
            safeToLogDownloadUrl.search = "...";
        if (safeToLogDownloadUrl.hash && safeToLogDownloadUrl.hash.length > 0)
            safeToLogDownloadUrl.hash = "...";
        return safeToLogDownloadUrl.toString();
    }
    /** Download the specified checkpoint file. This only downloads the file and does not update the [[Checkpoint]] id. Use [IModelDb.open]($backend) instead if you want to get a usable checkpoint file.
     * This method does not work on the browser. Directory containing the Checkpoint file is created if it does not exist. If there is an error during download, any partially downloaded file is deleted from disk.
     * @param requestContext The client request context
     * @param checkpoint Checkpoint to download. This needs to include a download link. See [[CheckpointQuery.selectDownloadUrl]].
     * @param path Path where checkpoint file should be downloaded, including filename.
     * @param progressCallback Callback for tracking progress.
     * @param cancelRequest Request used to cancel download
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if one of the arguments is undefined or has an invalid value.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.NotSupportedInBrowser]($bentley) if called in a browser.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.FileHandlerNotSet]($bentley) if [[FileHandler]] instance was not set for [[IModelClient]].
     * @throws [[ResponseError]] if the checkpoint cannot be downloaded.
     */
    async download(requestContext, checkpoint, path, progressCallback, cancelRequest) {
        requestContext.enter();
        Errors_1.ArgumentCheck.defined("checkpoint", checkpoint);
        Errors_1.ArgumentCheck.defined("path", path);
        if (typeof window !== "undefined")
            throw Errors_1.IModelHubClientError.browser();
        if (!this._fileHandler)
            throw Errors_1.IModelHubClientError.fileHandler();
        if (!checkpoint.downloadUrl)
            throw Errors_1.IModelHubClientError.missingDownloadUrl("checkpoint");
        const checkpointForLog = new Checkpoint();
        Object.assign(checkpointForLog, checkpoint);
        checkpointForLog.downloadUrl = CheckpointHandler.getSafeUrlForLogging(checkpointForLog.downloadUrl);
        const perfLogger = new bentleyjs_core_1.PerfLogger("Downloading checkpoint", () => (Object.assign(Object.assign({}, checkpointForLog), { path, iModelId: checkpoint.fileId })));
        await this._fileHandler.downloadFile(requestContext, checkpoint.downloadUrl, path, parseInt(checkpoint.fileSize, 10), progressCallback, cancelRequest);
        requestContext.enter();
        perfLogger.dispose();
    }
}
exports.CheckpointHandler = CheckpointHandler;
//# sourceMappingURL=Checkpoints.js.map