"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module iModelHubClient
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CodeHandler = exports.CodeSequenceHandler = exports.CodeSequence = exports.CodeSequenceType = exports.CodeQuery = exports.ConflictingCodesError = exports.DefaultCodeUpdateOptionsProvider = exports.MultiCode = exports.HubCode = exports.CodeBase = exports.CodeState = void 0;
const deepAssign = require("deep-assign");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const itwin_client_1 = require("@bentley/itwin-client");
const IModelHubClientLoggerCategories_1 = require("../IModelHubClientLoggerCategories");
const Errors_1 = require("./Errors");
const loggerCategory = IModelHubClientLoggerCategories_1.IModelHubClientLoggerCategory.IModelHub;
/**
 * [Code]($common) state describes whether the code is currently in use or owned by a [[Briefcase]].
 * @internal
 */
var CodeState;
(function (CodeState) {
    /** Code with this state is not persisted in iModelHub. Code that is updated to 'Available' state is deleted from iModelHub. */
    CodeState[CodeState["Available"] = 0] = "Available";
    /** Code is reserved by the [[Briefcase]], no one else is allowed to change its state. */
    CodeState[CodeState["Reserved"] = 1] = "Reserved";
    /** Code is used in a [[ChangeSet]] committed to iModelHub. */
    CodeState[CodeState["Used"] = 2] = "Used";
    /** Retired Code can not be reserved or used. It can only be deleted and then reserved again */
    CodeState[CodeState["Retired"] = 3] = "Retired";
})(CodeState = exports.CodeState || (exports.CodeState = {}));
/** Base class for [Code]($common)s.
 * @internal
 */
class CodeBase extends itwin_client_1.WsgInstance {
}
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.CodeSpecId")
], CodeBase.prototype, "codeSpecId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.CodeScope")
], CodeBase.prototype, "codeScope", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.State")
], CodeBase.prototype, "state", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.CreatedDate")
], CodeBase.prototype, "createdDate", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.BriefcaseId")
], CodeBase.prototype, "briefcaseId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.QueryOnly")
], CodeBase.prototype, "queryOnly", void 0);
exports.CodeBase = CodeBase;
/**
 * Code instance. Codes ensure uniqueness of names in the file.
 * @internal
 */
let HubCode = class HubCode extends CodeBase {
};
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Value")
], HubCode.prototype, "value", void 0);
HubCode = __decorate([
    itwin_client_1.ECJsonTypeMap.classToJson("wsg", "iModelScope.Code", { schemaPropertyName: "schemaName", classPropertyName: "className" })
], HubCode);
exports.HubCode = HubCode;
/**
 * MultiCode: Data about codes grouped by CodeSpecId, State and Briefcase
 * @internal
 */
let MultiCode = class MultiCode extends CodeBase {
};
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Values")
], MultiCode.prototype, "values", void 0);
MultiCode = __decorate([
    itwin_client_1.ECJsonTypeMap.classToJson("wsg", "iModelScope.MultiCode", { schemaPropertyName: "schemaName", classPropertyName: "className" })
], MultiCode);
exports.MultiCode = MultiCode;
/**
 * Encodes part of the code to be used in URI
 * @param str Part of the code.
 * @returns Encoded part of a code.
 */
function encodeForCodeId(str) {
    return encodeURIComponent(str.replace(/-/g, "_2D_"))
        .replace(/~/g, "~7E")
        .replace(/\*/g, "~2A")
        .replace(/%/g, "~");
}
/**
 * Gets encoded instance id for a code to be used in an URI.
 * @param code Code to get instance id for.
 * @returns Encoded code instance id.
 */
function getCodeInstanceId(code) {
    if (!code || !code.codeSpecId || !code.codeScope || !code.value)
        return undefined;
    return `'${code.codeSpecId}-${encodeForCodeId(code.codeScope)}-${encodeForCodeId(code.value)}'`;
}
/**
 * Provider for default CodeUpdateOptions, used by CodeHandler to set defaults.
 * @internal
 */
class DefaultCodeUpdateOptionsProvider {
    /**  Creates an instance of DefaultRequestOptionsProvider and sets up the default options. */
    constructor() {
        this._defaultOptions = {
            codesPerRequest: 2000,
        };
    }
    /**
     * Augments options with the provider's default values. The options passed in override any defaults where necessary.
     * @param options Options that should be augmented.
     */
    async assignOptions(options) {
        const clonedOptions = Object.assign({}, options);
        deepAssign(options, this._defaultOptions);
        deepAssign(options, clonedOptions); // ensure the supplied options override the defaults
    }
}
exports.DefaultCodeUpdateOptionsProvider = DefaultCodeUpdateOptionsProvider;
/**
 * Error for conflicting [Code]($common)s. It contains an array of Codes that failed to acquire. This is returned when calling [[CodeHandler.update]] with [[CodeUpdateOptions.deniedCodes]] set to true.
 * @internal
 */
class ConflictingCodesError extends Errors_1.IModelHubError {
    /**
     * Create ConflictingCodesError from IModelHubError instance.
     * @param error IModelHubError to get error data from.
     * @returns Undefined if the error is not for a code conflict, otherwise newly created error instance.
     * @internal
     */
    static fromError(error) {
        if (error.errorNumber !== bentleyjs_core_1.IModelHubStatus.CodeReservedByAnotherBriefcase &&
            error.errorNumber !== bentleyjs_core_1.IModelHubStatus.ConflictsAggregate) {
            return undefined;
        }
        const result = new ConflictingCodesError(error.errorNumber);
        deepAssign(result, error);
        result.addCodes(error);
        return result;
    }
    /**
     * Amend this error instance with conflicting codes from another IModelHubError.
     * @param error Error to get additional conflicting codes from.
     * @internal
     */
    addCodes(error) {
        if (!error.data || !error.data.ConflictingCodes) {
            return;
        }
        if (!this.conflictingCodes) {
            this.conflictingCodes = [];
        }
        for (const value of error.data.ConflictingCodes) {
            const instance = { className: "Code", schemaName: "iModelScope", properties: value };
            const code = itwin_client_1.ECJsonTypeMap.fromJson(HubCode, "wsg", instance);
            if (code) {
                this.conflictingCodes.push(code);
            }
        }
    }
}
exports.ConflictingCodesError = ConflictingCodesError;
/**
 * Query object for getting [Code]($common)s. You can use this to modify the query. See [[CodeHandler.get]].
 * @internal
 */
class CodeQuery extends itwin_client_1.WsgQuery {
    /** Constructor that sets default page size. */
    constructor() {
        super();
        this._isMultiCodeQuery = true;
        this.pageSize(CodeQuery.defaultPageSize);
    }
    /**
     * Used by the handler to check whether codes in query can be grouped.
     * @internal
     */
    get isMultiCodeQuery() {
        return this._isMultiCodeQuery;
    }
    /**
     * Query Codes by [[Briefcase]] id.
     * @param briefcaseId Id of the Briefcase.
     * @returns This query.
     * @throws [[IModelHubError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if briefcaseId is undefined or has an invalid [[Briefcase]] id value.
     */
    byBriefcaseId(briefcaseId) {
        Errors_1.ArgumentCheck.validBriefcaseId("briefcaseId", briefcaseId);
        this.addFilter(`BriefcaseId+eq+${briefcaseId}`);
        return this;
    }
    /**
     * Query Codes by CodeSpec id.
     * @param codeSpecId Id of the CodeSpec.
     * @returns This query.
     * @throws [[IModelHubError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) if codeSpecId is undefined or empty.
     */
    byCodeSpecId(codeSpecId) {
        Errors_1.ArgumentCheck.defined("codeSpecId", codeSpecId);
        this.addFilter(`CodeSpecId+eq+'${codeSpecId}'`);
        return this;
    }
    /**
     * Query Codes by Code scope.
     * @param codeScope Scope of the Code.
     * @returns This query.
     * @throws [[IModelHubError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) if codeScope is undefined or empty.
     */
    byCodeScope(codeScope) {
        Errors_1.ArgumentCheck.defined("codeScope", codeScope);
        this.addFilter(`CodeScope+eq+'${codeScope}'`);
        return this;
    }
    /**
     * Query [Code]($common)s by their instance ids.
     * @param codes Codes to query. They must have their codeSpec, scope and value set.
     * @returns This query.
     * @throws [[IModelHubError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if codes array is undefined, empty or it contains invalid [Code]($common) values.
     */
    byCodes(codes) {
        Errors_1.ArgumentCheck.nonEmptyArray("codes", codes);
        this._isMultiCodeQuery = false;
        if (codes.length < 1) {
            throw Errors_1.IModelHubClientError.invalidArgument("codes");
        }
        let filter = "$id+in+[";
        let index = 0;
        for (const code of codes) {
            const id = getCodeInstanceId(code);
            Errors_1.ArgumentCheck.valid(`codes[${index}]`, id);
            if (0 !== index++)
                filter += ",";
            filter += id;
        }
        filter += "]";
        this.addFilter(filter);
        return this;
    }
    /**
     * Query unavailable [Code]($common)s. It will include all Codes owned by other [[Briefcase]]s.
     * @param briefcaseId Id of the Briefcase.
     * @returns This query.
     * @throws [[IModelHubError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if briefcaseId is undefined or has an invalid [[Briefcase]] id value.
     */
    unavailableCodes(briefcaseId) {
        Errors_1.ArgumentCheck.validBriefcaseId("briefcaseId", briefcaseId);
        const filter = `BriefcaseId+ne+${briefcaseId}`;
        this.addFilter(filter);
        return this;
    }
}
exports.CodeQuery = CodeQuery;
/**
 * Default page size which is used when querying Codes
 * @internal
 */
CodeQuery.defaultPageSize = 10000;
/** Type of [[CodeSequence]] results.
 * @internal
 */
var CodeSequenceType;
(function (CodeSequenceType) {
    /** Return largest already used value. */
    CodeSequenceType[CodeSequenceType["LargestUsed"] = 0] = "LargestUsed";
    /** Return next available value in the sequence. */
    CodeSequenceType[CodeSequenceType["NextAvailable"] = 1] = "NextAvailable";
})(CodeSequenceType = exports.CodeSequenceType || (exports.CodeSequenceType = {}));
/** Sequence of [Code]($common)s matching a pattern. This class allows getting next available index based [Code]($common)
 * @internal
 */
let CodeSequence = class CodeSequence extends itwin_client_1.WsgInstance {
};
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.CodeSpecId")
], CodeSequence.prototype, "codeSpecId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.CodeScope")
], CodeSequence.prototype, "codeScope", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.ValuePattern")
], CodeSequence.prototype, "valuePattern", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Value")
], CodeSequence.prototype, "value", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.StartIndex")
], CodeSequence.prototype, "startIndex", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.IncrementBy")
], CodeSequence.prototype, "incrementBy", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Type")
], CodeSequence.prototype, "type", void 0);
CodeSequence = __decorate([
    itwin_client_1.ECJsonTypeMap.classToJson("wsg", "iModelScope.CodeSequence", { schemaPropertyName: "schemaName", classPropertyName: "className" })
], CodeSequence);
exports.CodeSequence = CodeSequence;
/**
 * Handler for querying [[CodeSequence]]s. Use [[CodeHandler.Sequences]] to get an instance of this class.
 * @internal
 */
class CodeSequenceHandler {
    /**
     * Constructor for CodeHandler.
     * @param handler Handler for WSG requests.
     * @internal
     */
    constructor(handler) {
        this._handler = handler;
    }
    /** Get relative url for Code sequence requests.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     */
    getRelativeUrl(iModelId) {
        return `/Repositories/iModel--${iModelId}/iModelScope/CodeSequence/`;
    }
    /** Get an index value based on the [[CodeSequence]]. This only suggests the last used or next available index value in the sequence and does not reserve the Code.
     * @param requestContext The client request context
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param sequence Code sequence describing the format of the Code value.
     * @returns Resolves to the suggested index value.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async get(requestContext, iModelId, sequence) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Querying code sequence for iModel", () => ({ iModelId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        const result = await this._handler.postInstance(requestContext, CodeSequence, this.getRelativeUrl(iModelId), sequence);
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Queried code sequence for iModel", () => ({ iModelId }));
        return result.value;
    }
}
exports.CodeSequenceHandler = CodeSequenceHandler;
/**
 * Handler for managing [Code]($common)s. Use [[IModelClient.Codes]] to get an instance of this class. In most cases, you should use [ConcurrencyControl]($backend) methods instead. You can read more about concurrency control [here]($docs/learning/backend/concurrencycontrol).
 * @internal
 */
class CodeHandler {
    /**
     * Constructor for CodeHandler.
     * @param handler Handler for WSG requests.
     * @internal
     */
    constructor(handler) {
        this._handler = handler;
    }
    /** Get handler for querying [[CodeSequence]]s. */
    get sequences() {
        return new CodeSequenceHandler(this._handler);
    }
    /** Get relative url for Code requests.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param codeId Id of the code.
     */
    getRelativeUrl(iModelId, multiCode = true, codeId) {
        return `/Repositories/iModel--${iModelId}/iModelScope/${multiCode ? "MultiCode" : "Code"}/${codeId || ""}`;
    }
    /** Convert Codes to MultiCodes. */
    static convertCodesToMultiCodes(codes) {
        const map = new Map();
        for (const code of codes) {
            const id = `${code.codeScope}-${code.codeSpecId}-${code.state}`;
            if (map.has(id)) {
                map.get(id).values.push(code.value);
            }
            else {
                const multiCode = new MultiCode();
                multiCode.changeState = "new";
                multiCode.briefcaseId = code.briefcaseId;
                multiCode.codeScope = code.codeScope;
                multiCode.codeSpecId = code.codeSpecId;
                multiCode.state = code.state;
                multiCode.values = [code.value];
                map.set(id, multiCode);
            }
        }
        return Array.from(map.values());
    }
    /** Convert MultiCodes to Codes. */
    static convertMultiCodesToCodes(multiCodes) {
        const result = [];
        for (const multiCode of multiCodes) {
            for (const value of multiCode.values) {
                const code = new HubCode();
                code.briefcaseId = multiCode.briefcaseId;
                code.codeScope = multiCode.codeScope;
                code.codeSpecId = multiCode.codeSpecId;
                code.state = multiCode.state;
                code.value = value;
                result.push(code);
            }
        }
        return result;
    }
    /**
     * Augment update options with defaults returned by the DefaultCodeUpdateOptionsProvider. The options passed in by clients override any defaults where necessary.
     * @param options Options the caller wants to augment with the defaults.
     * @returns Promise resolves after the defaults are setup.
     */
    async setupOptionDefaults(options) {
        if (!CodeHandler._defaultUpdateOptionsProvider)
            CodeHandler._defaultUpdateOptionsProvider = new DefaultCodeUpdateOptionsProvider();
        return CodeHandler._defaultUpdateOptionsProvider.assignOptions(options);
    }
    /** Send partial request for code updates */
    async updateInternal(requestContext, iModelId, codes, updateOptions) {
        requestContext.enter();
        let requestOptions;
        if (updateOptions) {
            requestOptions = {};
            requestOptions.CustomOptions = {};
            if (updateOptions.deniedCodes === false) {
                requestOptions.CustomOptions.DetailedError_Codes = "false"; // eslint-disable-line @typescript-eslint/naming-convention
            }
            if (updateOptions.unlimitedReporting) {
                requestOptions.CustomOptions.DetailedError_MaximumInstances = "-1"; // eslint-disable-line @typescript-eslint/naming-convention
            }
            if (updateOptions.continueOnConflict) {
                requestOptions.CustomOptions.ConflictStrategy = "Continue";
            }
            if (Object.getOwnPropertyNames(requestOptions.CustomOptions).length === 0)
                requestOptions = undefined;
        }
        const result = await this._handler.postInstances(requestContext, MultiCode, `/Repositories/iModel--${iModelId}/$changeset`, CodeHandler.convertCodesToMultiCodes(codes), requestOptions);
        return CodeHandler.convertMultiCodesToCodes(result);
    }
    /**
     * Update multiple [Code]($common)s. This call can simultaneously reserve new Codes and update states of already owned Codes. If large amount of Codes are updated, they are split across multiple requests. See [[CodeUpdateOptions.codesPerRequest]]. Default is 2000 Codes per request.
     * @param requestContext The client request context
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param codes Codes to update. Requires briefcaseId, state, codeSpecId, codeScope and value to be set on every instance. briefcaseId must be the same for every Code. Set queryOnly to true to just check if a Code can be reserved.
     * @param updateOptions Options for the update request. You can set this to change how conflicts are handled or to handle different amount of Codes per request.
     * @returns The code that was just obtained from the server.
     * @throws [[ConflictingCodesError]] when [[CodeUpdateOptions.deniedCodes]] is set and conflicts occurred. See [Handling Conflicts]($docs/learning/iModelHub/CodesAndLocksConflicts.md) for more information.
     * @throws [[AggregateResponseError]] when multiple requests where sent and more than 1 of the following errors occurred.
     * @throws [[IModelHubError]] with status indicating a conflict. See [Handling Conflicts]($docs/learning/iModelHub/CodesAndLocksConflicts.md) section for more information.
     * @throws [[IModelHubError]] with [IModelHubStatus.InvalidBriefcase]($bentley) when including Codes with different briefcaseId values in the request.
     * @throws [[IModelHubError]] with [IModelHubStatus.OperationFailed]($bentley) when including multiple identical Codes in the request.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async update(requestContext, iModelId, codes, updateOptions) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Requesting codes for iModel", () => ({ iModelId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        Errors_1.ArgumentCheck.nonEmptyArray("codes", codes);
        updateOptions = updateOptions || {};
        await this.setupOptionDefaults(updateOptions);
        const result = [];
        let conflictError;
        const aggregateError = new Errors_1.AggregateResponseError();
        for (let i = 0; i < codes.length; i += updateOptions.codesPerRequest) {
            const chunk = codes.slice(i, i + updateOptions.codesPerRequest);
            try {
                result.push(...await this.updateInternal(requestContext, iModelId, chunk, updateOptions));
                requestContext.enter();
            }
            catch (error) {
                requestContext.enter();
                if (error instanceof itwin_client_1.ResponseError) {
                    if (updateOptions && updateOptions.deniedCodes && error instanceof Errors_1.IModelHubError && (error.errorNumber === bentleyjs_core_1.IModelHubStatus.CodeReservedByAnotherBriefcase ||
                        error.errorNumber === bentleyjs_core_1.IModelHubStatus.ConflictsAggregate)) {
                        if (conflictError) {
                            conflictError.addCodes(error);
                        }
                        else {
                            conflictError = ConflictingCodesError.fromError(error);
                        }
                        if (!updateOptions.continueOnConflict) {
                            throw conflictError;
                        }
                    }
                    else {
                        aggregateError.errors.push(error);
                    }
                }
            }
        }
        if (conflictError) {
            throw conflictError;
        }
        if (aggregateError.errors.length > 0) {
            throw aggregateError.errors.length > 1 ? aggregateError : aggregateError.errors[0];
        }
        bentleyjs_core_1.Logger.logTrace(loggerCategory, `Requested ${codes.length} codes for iModel`, () => ({ iModelId }));
        return result;
    }
    /**
     * Get the [Code]($common)s that have been issued for the iModel.
     * @param requestContext The client request context
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param query Optional query object to filter the queried Codes or select different data from them.
     * @returns Resolves to an array of Codes matching the query.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async get(requestContext, iModelId, query = new CodeQuery()) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Querying codes for iModel", () => ({ iModelId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        let codes;
        if (query.isMultiCodeQuery) {
            const multiCodes = await this._handler.getInstances(requestContext, MultiCode, this.getRelativeUrl(iModelId), query.getQueryOptions());
            requestContext.enter();
            codes = CodeHandler.convertMultiCodesToCodes(multiCodes);
        }
        else {
            codes = await this._handler.postQuery(requestContext, HubCode, this.getRelativeUrl(iModelId, false), query.getQueryOptions());
            requestContext.enter();
        }
        bentleyjs_core_1.Logger.logTrace(loggerCategory, `Queried ${codes.length} codes for iModel`, () => ({ iModelId }));
        return codes;
    }
    /** Delete all [Code]($common)s owned by the specified [[Briefcase]].
     * @param requestContext The client request context
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param briefcaseId Id of the Briefcase.
     * @throws [[IModelHubError]] with [IModelHubStatus.BriefcaseDoesNotExist]($bentley) if [[Briefcase]] with specified briefcaseId does not exist. This can happen if number was not given as a Briefcase id yet, or Briefcase with that id was already deleted.
     * @throws [[IModelHubError]] with [IModelHubStatus.UserDoesNotHavePermission]($bentley) if [[Briefcase]] belongs to another user and user sending the request does not have ManageResources permission.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async deleteAll(requestContext, iModelId, briefcaseId) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Deleting all codes from briefcase", () => ({ briefcaseId, iModelId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        Errors_1.ArgumentCheck.validBriefcaseId("briefcaseId", briefcaseId);
        await this._handler.delete(requestContext, this.getRelativeUrl(iModelId, false, `DiscardReservedCodes-${briefcaseId}`));
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Deleted all codes from briefcase", () => ({ briefcaseId, iModelId }));
    }
}
exports.CodeHandler = CodeHandler;
//# sourceMappingURL=Codes.js.map