"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module iModelHubClient
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.AggregateResponseError = exports.ArgumentCheck = exports.IModelHubClientError = exports.IModelHubError = void 0;
const deepAssign = require("deep-assign");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const itwin_client_1 = require("@bentley/itwin-client");
const IModelHubClientLoggerCategories_1 = require("../IModelHubClientLoggerCategories");
const loggerCategory = IModelHubClientLoggerCategories_1.IModelHubClientLoggerCategory.IModelHub;
/**
 * Error returned from iModelHub service.
 * @public
 */
class IModelHubError extends itwin_client_1.WsgError {
    /** @internal */
    constructor(errorNumber, message, getMetaData) {
        super(errorNumber, message, getMetaData);
    }
    /**
     * Get IModelHubStatus from the string value returned by the service.
     * @param error Error id returned by the service.
     * @returns IModelHubStatus id
     */
    static getErrorId(error) {
        const id = bentleyjs_core_1.IModelHubStatus[error.slice(IModelHubError._idPrefix.length)];
        return id ? id : bentleyjs_core_1.IModelHubStatus.Unknown;
    }
    /**
     * Check whether error could have extended data.
     * @param id Id of the error.
     * @returns True if service can return extended data for this error id.
     */
    static requiresExtendedData(id) {
        switch (id) {
            case bentleyjs_core_1.IModelHubStatus.LockOwnedByAnotherBriefcase:
            case bentleyjs_core_1.IModelHubStatus.iModelAlreadyExists:
            case bentleyjs_core_1.IModelHubStatus.FileAlreadyExists:
            case bentleyjs_core_1.IModelHubStatus.PullIsRequired:
            case bentleyjs_core_1.IModelHubStatus.CodeStateInvalid:
            case bentleyjs_core_1.IModelHubStatus.CodeReservedByAnotherBriefcase:
            case bentleyjs_core_1.IModelHubStatus.ConflictsAggregate:
                return true;
            default:
                return false;
        }
    }
    /**
     * Make extended data available publically.
     */
    copyExtendedData() {
        this.data = this._data;
    }
    /**
     * Create IModelHubError from id.
     * @param id Id of the error.
     * @param message Message for the error.
     * @returns Created error.
     * @internal
     */
    static fromId(id, message) {
        const error = new IModelHubError(id);
        error.name = bentleyjs_core_1.IModelHubStatus[id];
        error.message = message;
        return error;
    }
    /**
     * Attempt to parse IModelHubError from server response.
     * @param response Response from the server.
     * @returns Parsed error.
     * @internal
     */
    static parse(response, log = true) {
        const wsgError = super.parse(response, false);
        if (wsgError instanceof itwin_client_1.WsgError && wsgError.name && wsgError.name.startsWith(IModelHubError._idPrefix)) {
            const errorId = IModelHubError.getErrorId(wsgError.name);
            const error = new IModelHubError(errorId);
            deepAssign(error, wsgError);
            error.errorNumber = errorId;
            if (IModelHubError.requiresExtendedData(error.errorNumber)) {
                error.copyExtendedData();
            }
            if (log)
                error.log();
            return error;
        }
        if (log)
            wsgError.log();
        return wsgError;
    }
    /**
     * Decides whether request should be retried or not.
     * @param error Error returned by request
     * @param response Response returned by request
     * @internal
     */
    static shouldRetry(error, response) {
        if (response === undefined || response === null) {
            return super.shouldRetry(error, response);
        }
        if (super.parseHttpStatus(response.statusType) === bentleyjs_core_1.HttpStatus.Success) {
            return false;
        }
        const parsedError = IModelHubError.parse({ response }, false);
        if (!(parsedError instanceof itwin_client_1.WsgError)) {
            return super.shouldRetry(error, response);
        }
        if ((parsedError instanceof IModelHubError)) {
            return false;
        }
        const errorCodesToRetry = [
            bentleyjs_core_1.HttpStatus.ServerError,
            bentleyjs_core_1.WSStatus.Unknown,
        ];
        const errorStatus = super.getErrorStatus(parsedError.name !== undefined ?
            super.getWSStatusId(parsedError.name) : bentleyjs_core_1.WSStatus.Unknown, response.statusType);
        return errorCodesToRetry.includes(errorStatus);
    }
    /**
     * Get log function.
     * @internal
     */
    getLogLevel() {
        switch (this.errorNumber) {
            case bentleyjs_core_1.IModelHubStatus.AnotherUserPushing:
            case bentleyjs_core_1.IModelHubStatus.PullIsRequired:
            case bentleyjs_core_1.IModelHubStatus.LockOwnedByAnotherBriefcase:
            case bentleyjs_core_1.IModelHubStatus.CodeReservedByAnotherBriefcase:
                return bentleyjs_core_1.Logger.logWarning;
            default:
                return bentleyjs_core_1.Logger.logError;
        }
    }
    /**
     * Logs this error.
     * @internal
     */
    log() {
        (this.getLogLevel())(loggerCategory, this.logMessage(), this.getMetaData());
    }
}
exports.IModelHubError = IModelHubError;
IModelHubError._idPrefix = "iModelHub.";
/**
 * Errors for incorrect iModelHub requests.
 * @public
 */
class IModelHubClientError extends IModelHubError {
    /** Creates IModelHubClientError from id.
     * @param id Id of the error.
     * @param message Message for the error.
     * @returns Created error.
     * @internal
     */
    static fromId(id, message) {
        const error = new IModelHubClientError(id, message);
        error.log();
        return error;
    }
    /** Create error for undefined arguments being passed.
     * @param argumentName Undefined argument name
     * @returns Created error.
     * @internal
     */
    static undefinedArgument(argumentName) {
        return this.fromId(bentleyjs_core_1.IModelHubStatus.UndefinedArgumentError, `Argument ${argumentName} is null or undefined`);
    }
    /** Create error for invalid arguments being passed.
     * @param argumentName Invalid argument name
     * @returns Created error.
     * @internal
     */
    static invalidArgument(argumentName) {
        return this.fromId(bentleyjs_core_1.IModelHubStatus.InvalidArgumentError, `Argument ${argumentName} has an invalid value.`);
    }
    /** Create error for arguments being passed that are missing download URL.
     * @param argumentName Argument name
     * @returns Created error.
     * @internal
     */
    static missingDownloadUrl(argumentName) {
        return this.fromId(bentleyjs_core_1.IModelHubStatus.MissingDownloadUrlError, `Supplied ${argumentName} must include download URL. Use selectDownloadUrl() when getting ${argumentName}.`);
    }
    /** Create error for seed file initialization timing out.
     * @returns Created error.
     * @internal
     */
    static initializationTimeout() {
        return this.fromId(bentleyjs_core_1.IModelHubStatus.InitializationTimeout, `Timed out waiting for Seed File initialization.`);
    }
    /** Create error for incompatible operation being used in browser.
     * @returns Created error.
     * @internal
     */
    static browser() {
        return this.fromId(bentleyjs_core_1.IModelHubStatus.NotSupportedInBrowser, "Operation is not supported in browser.");
    }
    /** Create error for incompatible operation being used in browser.
     * @returns Created error.
     * @internal
     */
    static fileHandler() {
        return this.fromId(bentleyjs_core_1.IModelHubStatus.FileHandlerNotSet, "File handler is required to be set for file download / upload.");
    }
    /** Create error for a missing file.
     * @returns Created error.
     * @internal
     */
    static fileNotFound() {
        return this.fromId(bentleyjs_core_1.IModelHubStatus.FileNotFound, "Could not find the file to upload.");
    }
}
exports.IModelHubClientError = IModelHubClientError;
/** @internal */
class ArgumentCheck {
    static defined(argumentName, argument, allowEmpty = false) {
        if (argument === undefined || argument === null || (argument === "" && !allowEmpty))
            throw IModelHubClientError.undefinedArgument(argumentName);
    }
    static definedNumber(argumentName, argument) {
        if (typeof argument !== "number")
            throw IModelHubClientError.undefinedArgument(argumentName);
    }
    static valid(argumentName, argument) {
        if (!argument)
            throw IModelHubClientError.invalidArgument(argumentName);
    }
    static validGuid(argumentName, argument) {
        this.defined(argumentName, argument);
        if (!bentleyjs_core_1.Guid.isGuid(argument))
            throw IModelHubClientError.invalidArgument(argumentName);
    }
    static nonEmptyArray(argumentName, argument) {
        this.defined(argumentName, argument);
        if (argument.length < 1)
            throw IModelHubClientError.invalidArgument(argumentName);
    }
    /** Check if Briefcase Id is valid. */
    static isBriefcaseIdValid(briefcaseId) {
        return briefcaseId > 1 && briefcaseId < 16 * 1024 * 1024;
    }
    /** Check if Briefcase Id argument is valid. */
    static validBriefcaseId(argumentName, argument) {
        this.definedNumber(argumentName, argument);
        if (!this.isBriefcaseIdValid(argument))
            throw IModelHubClientError.invalidArgument(argumentName);
    }
    static isValidChangeSetId(changeSetId, allowEmpty = false) {
        if (changeSetId.length === 0)
            return allowEmpty;
        const pattern = new RegExp("^[0-9A-Fa-f]+$");
        return changeSetId.length === 40 && pattern.test(changeSetId);
    }
    static validChangeSetId(argumentName, argument, allowEmpty = false) {
        this.defined(argumentName, argument, allowEmpty);
        if (!this.isValidChangeSetId(argument, allowEmpty))
            throw IModelHubClientError.invalidArgument(argumentName);
    }
}
exports.ArgumentCheck = ArgumentCheck;
/** Class for aggregating errors from multiple requests. Only thrown when more than 1 error has occurred.
 * @public
 */
class AggregateResponseError extends Error {
    constructor() {
        super(...arguments);
        /** Errors that happened over multiple requests. */
        this.errors = [];
    }
}
exports.AggregateResponseError = AggregateResponseError;
//# sourceMappingURL=Errors.js.map