"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module iModelHubClient
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventListener = exports.ListenerSubscription = exports.EventBaseHandler = exports.getEventBaseOperationRequestOptions = exports.GetEventOperationToRequestType = exports.ModifyEventOperationToRequestType = exports.IModelHubBaseEvent = exports.BaseEventSAS = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const itwin_client_1 = require("@bentley/itwin-client");
/**
 * Base class for event shared access signatures.
 * @public
 */
class BaseEventSAS extends itwin_client_1.WsgInstance {
}
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.BaseAddress")
], BaseEventSAS.prototype, "baseAddress", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.EventServiceSASToken")
], BaseEventSAS.prototype, "sasToken", void 0);
exports.BaseEventSAS = BaseEventSAS;
/** Base type for all iModelHub global events
 * @public
 */
class IModelHubBaseEvent {
    /**
     * Constructor for an event to pass members required for non-destructive reads.
     * @param handler Base handler for WSG requests.
     * @param sasToken Token for authenticating for event requests.
     * @internal
     */
    constructor(handler, sasToken) {
        this._handler = handler;
        this._sasToken = sasToken;
    }
    /**
     * Construct this event from object instance.
     * @param obj Object instance.
     * @internal
     */
    fromJson(obj) {
        this.eventTopic = obj.EventTopic;
        this.fromEventSubscriptionId = obj.FromEventSubscriptionId;
        this.toEventSubscriptionId = obj.ToEventSubscriptionId;
        this._lockUrl = obj.location;
    }
    /**
     * Remove a single event from queue.
     * @returns true if operation succeeded, false otherwise.
     */
    async delete(requestContext) {
        if (this._handler && this._lockUrl && this._sasToken) {
            const options = await getEventBaseOperationRequestOptions(this._handler, ModifyEventOperationToRequestType.Delete, this._sasToken);
            const result = await itwin_client_1.request(requestContext, this._lockUrl, options);
            if (result.status === 200)
                return true;
        }
        return false;
    }
}
exports.IModelHubBaseEvent = IModelHubBaseEvent;
/** @internal */
var ModifyEventOperationToRequestType;
(function (ModifyEventOperationToRequestType) {
    /** Deleted event from queue */
    ModifyEventOperationToRequestType["Delete"] = "DELETE";
})(ModifyEventOperationToRequestType = exports.ModifyEventOperationToRequestType || (exports.ModifyEventOperationToRequestType = {}));
/** @internal */
var GetEventOperationToRequestType;
(function (GetEventOperationToRequestType) {
    /** Get event request options, destructive get. */
    GetEventOperationToRequestType["GetDestructive"] = "DELETE";
    /** Get event request options, non destructive get. */
    GetEventOperationToRequestType["GetPeek"] = "POST";
})(GetEventOperationToRequestType = exports.GetEventOperationToRequestType || (exports.GetEventOperationToRequestType = {}));
/**
 * Get base request options for event operations.
 * @param method Method for request.
 * @param sasToken Service Bus SAS Token.
 * @param requestTimeout Timeout for the request.
 * @return Event if it exists.
 * @internal
 */
async function getEventBaseOperationRequestOptions(handler, method, sasToken, requestTimeout) {
    const options = {
        method,
        headers: { authorization: sasToken },
        agent: handler.getAgent(),
        retries: 0,
    };
    // Request timeout is in seconds, wait 50% more than the expected timeout from server
    if (requestTimeout)
        options.timeout = {
            deadline: requestTimeout * 1500,
            response: requestTimeout * 1500,
        };
    await new itwin_client_1.DefaultRequestOptionsProvider().assignOptions(options);
    return options;
}
exports.getEventBaseOperationRequestOptions = getEventBaseOperationRequestOptions;
/** @internal */
class EventBaseHandler {
    /** Get service bus parser depending on the environment. */
    setServiceBusOptions(options) {
        const parse = (message) => {
            if (!message)
                return undefined;
            return JSON.parse(message.substring(message.indexOf("{"), message.lastIndexOf("}") + 1));
        };
        if (typeof window !== "undefined") {
            options.parser = (_, message) => parse(message);
        }
        else {
            options.buffer = true;
            options.parser = (res, cb) => {
                res.on("data", (chunk) => { res.text += chunk; });
                res.on("end", () => {
                    try {
                        if (res.statusCode === 200 || res.statusCode === 201) {
                            cb(null, parse(res.text));
                        }
                        else if (res.statusCode === 204) {
                            cb(null, "");
                        }
                        else {
                            cb(res, null);
                        }
                    }
                    catch (err) {
                        cb(err, null);
                    }
                });
            };
        }
    }
    /**
     * Get event request options, gets event from queue.
     * @param sasToken Service Bus SAS Token.
     * @param requestTimeout Timeout for the request.
     * @return Event if it exists.
     */
    async getEventRequestOptions(operation, sasToken, requestTimeout) {
        const options = await getEventBaseOperationRequestOptions(this._handler, operation, sasToken, requestTimeout);
        this.setServiceBusOptions(options);
        return options;
    }
}
exports.EventBaseHandler = EventBaseHandler;
/** @internal */
class ListenerSubscription {
}
exports.ListenerSubscription = ListenerSubscription;
/** @internal */
class EventListener {
    /** @internal */
    static create(subscription, listener) {
        if (!this._subscriptions) {
            this._subscriptions = new Map();
        }
        let existingSubscription = this._subscriptions.get(subscription.id);
        let deleteListener;
        if (!existingSubscription) {
            existingSubscription = subscription;
            existingSubscription.listeners = new bentleyjs_core_1.BeEvent();
            deleteListener = subscription.listeners.addListener(listener);
            this.getEvents(subscription); // eslint-disable-line @typescript-eslint/no-floating-promises
        }
        else {
            deleteListener = subscription.listeners.addListener(listener);
        }
        this._subscriptions.set(subscription.id, existingSubscription);
        const subscriptionId = subscription.id;
        return () => {
            deleteListener();
            const sub = this._subscriptions.get(subscriptionId);
            if (sub) {
                if (sub.listeners && sub.listeners.numberOfListeners === 0)
                    this._subscriptions.delete(subscription.id);
            }
        };
    }
    static async getEvents(subscription) {
        let accessToken = await subscription.authenticationCallback();
        let eventSAS;
        mainLoop: while (subscription.listeners.numberOfListeners > 0) {
            try {
                const requestContext = new itwin_client_1.AuthorizedClientRequestContext(accessToken);
                eventSAS = (await subscription.getSASToken(requestContext));
            }
            catch (err) {
                if (err.status === 401) {
                    try {
                        accessToken = await subscription.authenticationCallback();
                    }
                    catch (_a) {
                        break;
                    }
                }
                else {
                    break;
                }
            }
            while (subscription.listeners.numberOfListeners > 0) {
                try {
                    const event = await subscription.getEvent(eventSAS.sasToken, eventSAS.baseAddress, subscription.id, 60);
                    if (event)
                        subscription.listeners.raiseEvent(event);
                }
                catch (err) {
                    if (err.status === 401) {
                        break;
                    }
                    else {
                        break mainLoop;
                    }
                }
            }
        }
        this._subscriptions.delete(subscription.id);
    }
}
exports.EventListener = EventListener;
//# sourceMappingURL=EventsBase.js.map