"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GlobalEventHandler = exports.GetEventOperationType = exports.GlobalEventSubscriptionHandler = exports.GlobalEventSAS = exports.GlobalEventSubscription = exports.ParseGlobalEvent = exports.GlobalCheckpointV2CreatedEvent = exports.GlobalCheckpointCreatedEvent = exports.NamedVersionCreatedEvent = exports.ChangeSetCreatedEvent = exports.IModelCreatedEvent = exports.HardiModelDeleteEvent = exports.SoftiModelDeleteEvent = exports.IModelHubGlobalEvent = void 0;
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module iModelHubClient
 */
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const context_registry_client_1 = require("@bentley/context-registry-client");
const itwin_client_1 = require("@bentley/itwin-client");
const IModelHubClientLoggerCategories_1 = require("../IModelHubClientLoggerCategories");
const Errors_1 = require("./Errors");
const EventsBase_1 = require("./EventsBase");
const loggerCategory = IModelHubClientLoggerCategories_1.IModelHubClientLoggerCategory.IModelHub;
/** Base type for all iModelHub global events.
 * @internal
 */
class IModelHubGlobalEvent extends EventsBase_1.IModelHubBaseEvent {
    /** Construct this global event from object instance.
     * @param obj Object instance.
     * @internal
     */
    fromJson(obj) {
        super.fromJson(obj);
        this.iModelId = obj.iModelId;
        this.projectId = obj.ProjectId;
        this.contextId = obj.ContextId;
        const contextTypeId = obj.ContextTypeId;
        switch (contextTypeId) {
            case context_registry_client_1.ContextType.Asset:
            case context_registry_client_1.ContextType.Project:
                this.contextTypeId = contextTypeId;
                break;
            default:
                this.contextTypeId = context_registry_client_1.ContextType.Unknown;
        }
    }
}
exports.IModelHubGlobalEvent = IModelHubGlobalEvent;
/** Sent when an iModel is put into the archive. See [[IModelHandler.delete]].
 * @internal Rename to SoftIModelDeleteEvent
 */
class SoftiModelDeleteEvent extends IModelHubGlobalEvent {
}
exports.SoftiModelDeleteEvent = SoftiModelDeleteEvent;
/** Sent when an archived iModel is completely deleted from the storage. Sent after some time passes after [[IModelHandler.delete]] and iModel is no longer kept in the archive. iModel is kept at least 30 days in the archive.
 * @internal Rename to HardIModelDeleteEvent
 */
class HardiModelDeleteEvent extends IModelHubGlobalEvent {
}
exports.HardiModelDeleteEvent = HardiModelDeleteEvent;
/** Sent when an iModel is created. See [[IModelHandler.create]].
 * @internal
 */
class IModelCreatedEvent extends IModelHubGlobalEvent {
}
exports.IModelCreatedEvent = IModelCreatedEvent;
/** Sent when a [[ChangeSet]] is pushed. See [[ChangeSetHandler.create]]. Sent together with [[ChangeSetPostPushEvent]].
 * @internal
 */
class ChangeSetCreatedEvent extends IModelHubGlobalEvent {
    /** Construct this event from object instance.
     * @param obj Object instance.
     */
    fromJson(obj) {
        super.fromJson(obj);
        this.changeSetId = obj.ChangeSetId;
        this.changeSetIndex = obj.ChangeSetIndex;
        this.briefcaseId = obj.BriefcaseId;
    }
}
exports.ChangeSetCreatedEvent = ChangeSetCreatedEvent;
/** Sent when a named [[Version]] is created. See [[VersionHandler.create]].
 * @internal
 */
class NamedVersionCreatedEvent extends IModelHubGlobalEvent {
    /** Construct this event from object instance.
     * @param obj Object instance.
     */
    fromJson(obj) {
        super.fromJson(obj);
        this.versionId = obj.VersionId;
        this.versionName = obj.VersionName;
        this.changeSetId = obj.ChangeSetId;
    }
}
exports.NamedVersionCreatedEvent = NamedVersionCreatedEvent;
/** Sent when a new [[Checkpoint]] is generated. [[Checkpoint]]s can be generated daily when there are new [[ChangeSet]]s pushed or when a new [[Version]] is created.
 * @internal
 */
class GlobalCheckpointCreatedEvent extends IModelHubGlobalEvent {
    /** Construct this event from object instance.
     * @param obj Object instance.
     */
    fromJson(obj) {
        super.fromJson(obj);
        this.changeSetIndex = obj.ChangeSetIndex;
        this.changeSetId = obj.ChangeSetId;
        this.versionId = obj.VersionId;
    }
}
exports.GlobalCheckpointCreatedEvent = GlobalCheckpointCreatedEvent;
/** Sent when a new [[CheckpointV2]] is generated. [[CheckpointV2]] might be created for every [[ChangeSet]].
 * @internal
 */
class GlobalCheckpointV2CreatedEvent extends IModelHubGlobalEvent {
    /** Construct this event from object instance.
     * @param obj Object instance.
     */
    fromJson(obj) {
        super.fromJson(obj);
        this.changeSetIndex = obj.ChangeSetIndex;
        this.changeSetId = obj.ChangeSetId;
        this.versionId = obj.VersionId;
    }
}
exports.GlobalCheckpointV2CreatedEvent = GlobalCheckpointV2CreatedEvent;
/** Get constructor from GlobalEventType name. */
function constructorFromEventType(type) {
    switch (type) {
        case "SoftiModelDeleteEvent":
            return SoftiModelDeleteEvent;
        case "HardiModelDeleteEvent":
            return HardiModelDeleteEvent;
        case "iModelCreatedEvent":
            return IModelCreatedEvent;
        case "ChangeSetCreatedEvent":
            return ChangeSetCreatedEvent;
        case "NamedVersionCreatedEvent":
            return NamedVersionCreatedEvent;
        case "CheckpointCreatedEvent":
            return GlobalCheckpointCreatedEvent;
        case "CheckpointV2CreatedEvent":
            return GlobalCheckpointV2CreatedEvent;
    }
}
/** Parse [[IModelHubGlobalEvent]] from response object.
 * @param response Response object to parse.
 * @returns Appropriate global event object.
 * @internal
 */
// eslint-disable-next-line @typescript-eslint/naming-convention
function ParseGlobalEvent(response, handler, sasToken) {
    const constructor = constructorFromEventType(response.header["content-type"]);
    const globalEvent = new constructor(handler, sasToken);
    globalEvent.fromJson(Object.assign(Object.assign({}, response.header), response.body));
    return globalEvent;
}
exports.ParseGlobalEvent = ParseGlobalEvent;
/** Subscription to receive [[IModelHubGlobalEvent]]s. Each subscription has a separate queue for events that it hasn't read yet. Global event subscriptions do not expire and must be deleted by the user. Use wsgId of this instance for the methods that require subscriptionId. See [[GlobalEventSubscriptionHandler]].
 * @internal
 */
let GlobalEventSubscription = class GlobalEventSubscription extends itwin_client_1.WsgInstance {
};
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.EventTypes")
], GlobalEventSubscription.prototype, "eventTypes", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.SubscriptionId")
], GlobalEventSubscription.prototype, "subscriptionId", void 0);
GlobalEventSubscription = __decorate([
    itwin_client_1.ECJsonTypeMap.classToJson("wsg", "GlobalScope.GlobalEventSubscription", { schemaPropertyName: "schemaName", classPropertyName: "className" })
], GlobalEventSubscription);
exports.GlobalEventSubscription = GlobalEventSubscription;
/** Shared access signature token for getting [[IModelHubGlobalEvent]]s. It's used to authenticate for [[GlobalEventHandler.getEvent]]. To receive an instance call [[GlobalEventHandler.getSASToken]].
 * @internal
 */
let GlobalEventSAS = class GlobalEventSAS extends EventsBase_1.BaseEventSAS {
};
GlobalEventSAS = __decorate([
    itwin_client_1.ECJsonTypeMap.classToJson("wsg", "GlobalScope.GlobalEventSAS", { schemaPropertyName: "schemaName", classPropertyName: "className" })
], GlobalEventSAS);
exports.GlobalEventSAS = GlobalEventSAS;
/** Handler for managing [[GlobalEventSubscription]]s.
 * Use [[GlobalEventHandler.Subscriptions]] to get an instance of this class.
 * @internal
 */
class GlobalEventSubscriptionHandler {
    /** Constructor for GlobalEventSubscriptionHandler.
     * @param handler Handler for WSG requests.
     * @internal
     */
    constructor(handler) {
        this._handler = handler;
    }
    /** Get relative url for GlobalEventSubscription requests.
     * @param instanceId Id of the subscription.
     */
    getRelativeUrl(instanceId) {
        return `/Repositories/Global--Global/GlobalScope/GlobalEventSubscription/${instanceId || ""}`;
    }
    /** Create a [[GlobalEventSubscription]]. You can use this to get or update the existing subscription instance, if you only have the original subscriptionId.
     * @param requestContext The client request context
     * @param subscriptionId Guid to be used by global event subscription. It will be a part of the resulting subscription id.
     * @param globalEvents Array of GlobalEventTypes to subscribe to.
     * @return Created GlobalEventSubscription instance.
     * @throws [[IModelHubError]] with [IModelHubStatus.EventSubscriptionAlreadyExists]($bentley) if [[GlobalEventSubscription]] already exists with the specified subscriptionId.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async create(requestContext, subscriptionId, globalEvents) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Creating global event subscription", () => ({ subscriptionId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("subscriptionId", subscriptionId);
        let subscription = new GlobalEventSubscription();
        subscription.eventTypes = globalEvents;
        subscription.subscriptionId = subscriptionId;
        subscription = await this._handler.postInstance(requestContext, GlobalEventSubscription, this.getRelativeUrl(), subscription);
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Created global event subscription", () => ({ subscriptionId }));
        return subscription;
    }
    /** Update a [[GlobalEventSubscription]]. Can change the [[GlobalEventType]]s specified in the subscription. Must be a valid subscription that was previously created with [[GlobalEventSubscriptionHandler.create]].
     * @param requestContext The client request context.
     * @param subscription Updated GlobalEventSubscription.
     * @return GlobalEventSubscription instance from iModelHub after update.
     * @throws [[IModelHubError]] with [IModelHubStatus.EventSubscriptionDoesNotExist]($bentley) if [[GlobalEventSubscription]] does not exist with the specified subscription.wsgId.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async update(requestContext, subscription) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, `Updating global event subscription with instance id: ${subscription.wsgId}`, () => ({ subscriptionId: subscription.subscriptionId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.defined("subscription", subscription);
        Errors_1.ArgumentCheck.validGuid("subscription.wsgId", subscription.wsgId);
        const updatedSubscription = await this._handler.postInstance(requestContext, GlobalEventSubscription, this.getRelativeUrl(subscription.wsgId), subscription);
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, `Updated global event subscription with instance id: ${subscription.wsgId}`, () => ({ subscriptionId: subscription.subscriptionId }));
        return updatedSubscription;
    }
    /** Delete a [[GlobalEventSubscription]].
     * @param requestContext The client request context.
     * @param subscriptionId WSG Id of the GlobalEventSubscription.
     * @returns Resolves if the GlobalEventSubscription has been successfully deleted.
     * @throws [[IModelHubError]] with [IModelHubStatus.EventSubscriptionDoesNotExist]($bentley) if GlobalEventSubscription does not exist with the specified subscription.wsgId.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async delete(requestContext, subscriptionId) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Deleting global event subscription", () => ({ subscriptionId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("subscriptionInstanceId", subscriptionId);
        await this._handler.delete(requestContext, this.getRelativeUrl(subscriptionId));
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Deleted global event subscription", () => ({ subscriptionId }));
    }
}
exports.GlobalEventSubscriptionHandler = GlobalEventSubscriptionHandler;
/** Type of [[GlobalEventHandler.getEvent]] operations.
 * @internal
 */
var GetEventOperationType;
(function (GetEventOperationType) {
    /** Event will be immediately removed from queue. */
    GetEventOperationType[GetEventOperationType["Destructive"] = 0] = "Destructive";
    /** Event will be locked instead of removed. It has to be later removed via [[IModelHubBaseEvent.delete]]. */
    GetEventOperationType[GetEventOperationType["Peek"] = 1] = "Peek";
})(GetEventOperationType = exports.GetEventOperationType || (exports.GetEventOperationType = {}));
/** Handler for receiving [[IModelHubGlobalEvent]]s.
 * Use [[IModelClient.GlobalEvents]] to get an instance of this class.
 * @internal
 */
class GlobalEventHandler extends EventsBase_1.EventBaseHandler {
    /** Constructor for GlobalEventHandler.
     * @param handler Handler for WSG requests.
     * @internal
     */
    constructor(handler) {
        super();
        this._handler = handler;
    }
    /** Get a handler for managing [[GlobalEventSubscription]]s. */
    get subscriptions() {
        if (!this._subscriptionHandler) {
            this._subscriptionHandler = new GlobalEventSubscriptionHandler(this._handler);
        }
        return this._subscriptionHandler;
    }
    /** Get relative url for GlobalEventSAS requests. */
    getGlobalEventSASRelativeUrl() {
        return `/Repositories/Global--Global/GlobalScope/GlobalEventSAS/`;
    }
    /** Get global event SAS Token. Used to authenticate for [[GlobalEventHandler.getEvent]].
     * @param requestContext The client request context
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async getSASToken(requestContext) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Getting global event SAS token");
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        const globalEventSAS = await this._handler.postInstance(requestContext, GlobalEventSAS, this.getGlobalEventSASRelativeUrl(), new GlobalEventSAS());
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Got global event SAS token");
        return globalEventSAS;
    }
    /** Get absolute url for global event requests.
     * @param baseAddress Base address for the serviceBus.
     * @param subscriptionId Id of the subscription instance.
     * @param timeout Optional timeout for long polling.
     */
    getGlobalEventUrl(baseAddress, subscriptionId, timeout) {
        let url = `${baseAddress}/Subscriptions/${subscriptionId}/messages/head`;
        if (timeout) {
            url = `${url}?timeout=${timeout}`;
        }
        return url;
    }
    /** Get an [[IModelHubGlobalEvent]] from the [[GlobalEventSubscription]]. You can use long polling timeout, to have requests return when events are available (or request times out), rather than returning immediately when no events are found.
     * @param requestContext The client request context
     * @param sasToken SAS Token used to authenticate. See [[GlobalEventSAS.sasToken]].
     * @param baseAddress Address for the events. See [[GlobalEventSAS.baseAddress]].
     * @param subscriptionId Id of the subscription to the topic. See [[GlobalEventSubscription]].
     * @param timeout Optional timeout duration in seconds for request, when using long polling.
     * @return IModelHubGlobalEvent if it exists, undefined otherwise.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if one of the arguments is undefined or has an invalid value.
     * @throws [[ResponseError]] if request has failed.
     */
    async getEvent(requestContext, sasToken, baseAddress, subscriptionId, timeout, getOperation = GetEventOperationType.Destructive) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Getting global event from subscription", () => ({ subscriptionId }));
        Errors_1.ArgumentCheck.defined("sasToken", sasToken);
        Errors_1.ArgumentCheck.defined("baseAddress", baseAddress);
        Errors_1.ArgumentCheck.defined("subscriptionInstanceId", subscriptionId);
        let options;
        if (getOperation === GetEventOperationType.Destructive)
            options = await this.getEventRequestOptions(EventsBase_1.GetEventOperationToRequestType.GetDestructive, sasToken, timeout);
        else if (getOperation === GetEventOperationType.Peek)
            options = await this.getEventRequestOptions(EventsBase_1.GetEventOperationToRequestType.GetPeek, sasToken, timeout);
        else // Unknown operation type.
            return undefined;
        const result = await itwin_client_1.request(requestContext, this.getGlobalEventUrl(baseAddress, subscriptionId, timeout), options);
        requestContext.enter();
        if (result.status === 204) {
            bentleyjs_core_1.Logger.logTrace(loggerCategory, "No events found on subscription", () => ({ subscriptionId }));
            return undefined;
        }
        const event = ParseGlobalEvent(result, this._handler, sasToken);
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Got Global Event from subscription", () => ({ subscriptionId }));
        return event;
    }
    /** Create a listener for long polling events from a [[GlobalEventSubscription]]. When event is received from the subscription, every registered listener callback is called. This continuously waits for events until all created listeners for that subscriptionInstanceId are deleted. [[GlobalEventSAS]] token expirations are handled automatically, [[AccessToken]] expiration is handled by calling authenticationCallback to get a new token.
     * @param authenticationCallback Callback used to get AccessToken. Only the first registered authenticationCallback for this subscriptionId will be used.
     * @param subscriptionInstanceId Id of GlobalEventSubscription.
     * @param listener Callback that is called when an [[IModelHubGlobalEvent]] is received.
     * @returns Function that deletes the created listener.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if one of the arguments is undefined or has an invalid value.
     */
    createListener(requestContext, authenticationCallback, subscriptionInstanceId, listener) {
        requestContext.enter();
        Errors_1.ArgumentCheck.defined("subscriptionInstanceId", subscriptionInstanceId);
        const subscription = new EventsBase_1.ListenerSubscription();
        subscription.authenticationCallback = authenticationCallback;
        subscription.getEvent = async (sasToken, baseAddress, id, timeout) => this.getEvent(requestContext, sasToken, baseAddress, id, timeout);
        subscription.getSASToken = async (reqContext) => this.getSASToken(reqContext);
        subscription.id = subscriptionInstanceId;
        return EventsBase_1.EventListener.create(subscription, listener);
    }
}
exports.GlobalEventHandler = GlobalEventHandler;
//# sourceMappingURL=GlobalEvents.js.map