"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module iModelHubClient
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.addSelectApplicationData = exports.addSelectContainerAccessKey = exports.addSelectFileAccessKey = exports.InstanceIdQuery = exports.StringIdQuery = void 0;
const itwin_client_1 = require("@bentley/itwin-client");
const Errors_1 = require("./Errors");
/** Query for instances with string based instance ids.
 * @public
 */
class StringIdQuery extends itwin_client_1.WsgQuery {
    /**
     * Query single instance by its id.
     * @param id Id of the instance to query.
     * @returns This query.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if id is undefined or it is not a valid [GuidString]($bentley) value.
     */
    byId(id) {
        this.checkValue(id);
        this._byId = id;
        this._query.$pageSize = undefined;
        return this;
    }
    /** @internal */
    checkValue(id) {
        Errors_1.ArgumentCheck.valid("id", id);
    }
    /**
     * Used by iModelHub handlers to get the id that is queried.
     * @internal
     */
    getId() {
        return this._byId;
    }
}
exports.StringIdQuery = StringIdQuery;
/** Query for instances with Guid based instance ids.
 * @public
 */
class InstanceIdQuery extends itwin_client_1.WsgQuery {
    /**
     * Query single instance by its id.
     * @param id Id of the instance to query.
     * @returns This query.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if id is undefined or it is not a valid [GuidString]($bentley) value.
     */
    byId(id) {
        Errors_1.ArgumentCheck.validGuid("id", id);
        this._byId = id;
        this._query.$pageSize = undefined;
        return this;
    }
    /**
     * Used by iModelHub handlers to get the id that is queried.
     * @internal
     */
    getId() {
        return this._byId;
    }
}
exports.InstanceIdQuery = InstanceIdQuery;
/**
 * Add select for the download URL to the query.
 * @internal
 */
function addSelectFileAccessKey(query) {
    if (!query.$select)
        query.$select = "*";
    const fileAccessKeySelector = "FileAccessKey-forward-AccessKey.DownloadURL";
    if (query.$select.indexOf(fileAccessKeySelector) === -1)
        query.$select += `,${fileAccessKeySelector}`;
}
exports.addSelectFileAccessKey = addSelectFileAccessKey;
/**
 * Add select for the ContainerAccessKey to the query that allows
 * to read/download iModel in blocks incrementally.
 * @internal
 */
function addSelectContainerAccessKey(query) {
    if (!query.$select)
        query.$select = "*";
    query.$select += ",HasContainer-forward-ContainerAccessKey.*";
}
exports.addSelectContainerAccessKey = addSelectContainerAccessKey;
/**
 * Add select for the application data to the query.
 * @internal
 */
function addSelectApplicationData(query) {
    if (!query.$select)
        query.$select = "*";
    query.$select += ",CreatedByApplication-forward-Application.*";
}
exports.addSelectApplicationData = addSelectApplicationData;
//# sourceMappingURL=HubQuery.js.map