"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module iModelHubClient
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LockHandler = exports.LockQuery = exports.MultiLock = exports.Lock = exports.LockBase = exports.ConflictingLocksError = exports.DefaultLockUpdateOptionsProvider = exports.LockLevel = exports.LockType = void 0;
const deepAssign = require("deep-assign");
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const itwin_client_1 = require("@bentley/itwin-client");
const IModelHubClientLoggerCategories_1 = require("../IModelHubClientLoggerCategories");
const Errors_1 = require("./Errors");
const loggerCategory = IModelHubClientLoggerCategories_1.IModelHubClientLoggerCategory.IModelHub;
/**
 * [[Lock]] type describes the kind of object that is locked.
 * @internal
 */
var LockType;
(function (LockType) {
    /** Lock for the entire file. This is a global Lock that can only be taken with objectId 1. */
    LockType[LockType["Db"] = 0] = "Db";
    /** Lock for a model. It should be acquired together with a [[LockLevel.Shared]] Db Lock. */
    LockType[LockType["Model"] = 1] = "Model";
    /** Lock for a single element. It should be acquired together with a [[LockLevel.Shared]] Model Lock. */
    LockType[LockType["Element"] = 2] = "Element";
    /** Lock used to change schemas. This is a global Lock that can only be taken with objectId 1. This lock cannot have [[LockLevel.Shared]]. */
    LockType[LockType["Schemas"] = 3] = "Schemas";
    /** Lock used to change CodeSpecs. This is a global Lock that can only be taken with objectId 1. This lock cannot have [[LockLevel.Shared]]. */
    LockType[LockType["CodeSpecs"] = 4] = "CodeSpecs";
})(LockType = exports.LockType || (exports.LockType = {}));
/**
 * [[Lock]] level describes how restrictive the Lock is.
 * @internal
 */
var LockLevel;
(function (LockLevel) {
    /** Lock is not owned. */
    LockLevel[LockLevel["None"] = 0] = "None";
    /** Lock can be owned by multiple [[Briefcase]]s. Shared Lock is usually acquired together with Locks for lower level objects that depend on this object. */
    LockLevel[LockLevel["Shared"] = 1] = "Shared";
    /** Lock can only be owned by a single briefcase. Exclusive Lock is required to modify model or element when using pessimistic concurrency. */
    LockLevel[LockLevel["Exclusive"] = 2] = "Exclusive";
})(LockLevel = exports.LockLevel || (exports.LockLevel = {}));
/**
 * Gets encoded instance id for a lock to be used in an URI.
 * @param lock Lock to get instance id for.
 * @returns Encoded lock instance id.
 * @internal
 */
function getLockInstanceId(lock) {
    if (!lock || lock.briefcaseId === undefined || lock.lockType === undefined || !lock.objectId)
        return undefined;
    return `${lock.lockType}-${lock.objectId}-${lock.briefcaseId}`;
}
/** Provider for default LockUpdateOptions, used by LockHandler to set defaults.
 * @internal
 */
class DefaultLockUpdateOptionsProvider {
    /** Creates an instance of DefaultRequestOptionsProvider and sets up the default options. */
    constructor() {
        this._defaultOptions = {
            locksPerRequest: 2000,
        };
    }
    /** Augments options with the provider's default values.
     * @note The options passed in override any defaults where necessary.
     * @param options Options that should be augmented.
     */
    async assignOptions(options) {
        const clonedOptions = Object.assign({}, options);
        deepAssign(options, this._defaultOptions);
        deepAssign(options, clonedOptions); // ensure the supplied options override the defaults
    }
}
exports.DefaultLockUpdateOptionsProvider = DefaultLockUpdateOptionsProvider;
/**
 * Error for conflicting [[Lock]]s. It contains an array of Locks that failed to acquire. This is returned when calling [[LockHandler.update]] with [[LockUpdateOptions.deniedLocks]] set to true.
 * @internal
 */
class ConflictingLocksError extends Errors_1.IModelHubError {
    /** Create ConflictingLocksError from IModelHubError instance.
     * @param error IModelHubError to get error data from.
     * @returns Undefined if the error is not for a lock conflict, otherwise newly created error instance.
     * @internal
     */
    static fromError(error) {
        if (error.errorNumber !== bentleyjs_core_1.IModelHubStatus.LockOwnedByAnotherBriefcase
            && error.errorNumber !== bentleyjs_core_1.IModelHubStatus.ConflictsAggregate) {
            return undefined;
        }
        const result = new ConflictingLocksError(error.errorNumber);
        deepAssign(result, error);
        result.addLocks(error);
        return result;
    }
    /**
     * Amends this error instance with conflicting locks from another IModelHubError.
     * @param error Error to get additional conflicting locks from.
     * @internal
     */
    addLocks(error) {
        if (!error.data || !error.data.ConflictingLocks) {
            return;
        }
        if (!this.conflictingLocks) {
            this.conflictingLocks = [];
        }
        for (const value of error.data.ConflictingLocks) {
            const instance = { className: "Lock", schemaName: "iModelScope", properties: value };
            const lock = itwin_client_1.ECJsonTypeMap.fromJson(Lock, "wsg", instance);
            if (lock) {
                this.conflictingLocks.push(lock);
            }
        }
    }
}
exports.ConflictingLocksError = ConflictingLocksError;
/**
 * Base class for [[Lock]]s.
 * @internal
 */
class LockBase extends itwin_client_1.WsgInstance {
}
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.LockType")
], LockBase.prototype, "lockType", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.LockLevel")
], LockBase.prototype, "lockLevel", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.BriefcaseId")
], LockBase.prototype, "briefcaseId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.SeedFileId")
], LockBase.prototype, "seedFileId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.ReleasedWithChangeSet")
], LockBase.prototype, "releasedWithChangeSet", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.ReleasedWithChangeSetIndex")
], LockBase.prototype, "releasedWithChangeSetIndex", void 0);
exports.LockBase = LockBase;
/**
 * Lock instance. When using pessimistic concurrency, locks ensure that only a single user can modify an object at a time.
 * @internal
 */
let Lock = class Lock extends LockBase {
};
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.ObjectId")
], Lock.prototype, "objectId", void 0);
Lock = __decorate([
    itwin_client_1.ECJsonTypeMap.classToJson("wsg", "iModelScope.Lock", { schemaPropertyName: "schemaName", classPropertyName: "className" })
], Lock);
exports.Lock = Lock;
/**
 * MultiLock: data about locks grouped by BriefcaseId, LockLevel and LockType.
 * @internal
 */
let MultiLock = class MultiLock extends LockBase {
};
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.ObjectIds")
], MultiLock.prototype, "objectIds", void 0);
MultiLock = __decorate([
    itwin_client_1.ECJsonTypeMap.classToJson("wsg", "iModelScope.MultiLock", { schemaPropertyName: "schemaName", classPropertyName: "className" })
], MultiLock);
exports.MultiLock = MultiLock;
/**
 * Query object for getting [[Lock]]s. You can use this to modify the [[LockHandler.get]] results.
 * @internal
 */
class LockQuery extends itwin_client_1.WsgQuery {
    /** Constructor that sets default page size. */
    constructor() {
        super();
        this._isMultiLockQuery = true;
        this.pageSize(LockQuery.defaultPageSize);
    }
    /**
     * Used by the handler to check whether locks in query can be grouped.
     * @internal
     */
    get isMultiLockQuery() {
        return this._isMultiLockQuery;
    }
    /**
     * Query [[Lock]]s by [[Briefcase]] id.
     * @param briefcaseId Id of the Briefcase.
     * @returns This query.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if briefcaseId is undefined or it contains an invalid [[Briefcase]] id value.
     */
    byBriefcaseId(briefcaseId) {
        Errors_1.ArgumentCheck.validBriefcaseId("briefcaseId", briefcaseId);
        this.addFilter(`BriefcaseId+eq+${briefcaseId}`);
        return this;
    }
    /**
     * Query [[Lock]]s by [[LockType]].
     * @param lockType Lock type to query.
     * @returns This query.
     */
    byLockType(lockType) {
        this.addFilter(`LockType+eq+${lockType}`);
        return this;
    }
    /**
     * Query [[Lock]]s by [[LockLevel]].
     * @param lockLevel Lock level to query.
     * @returns This query.
     */
    byLockLevel(lockLevel) {
        this.addFilter(`LockLevel+eq+${lockLevel}`);
        return this;
    }
    /**
     * Query [[Lock]]s by ObjectId.
     * @param objectId Id of the object.
     * @returns This query.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) if objectId is undefined.
     */
    byObjectId(objectId) {
        Errors_1.ArgumentCheck.defined("objectId", objectId);
        this._isMultiLockQuery = false;
        this.addFilter(`ObjectId+eq+'${objectId}'`);
        return this;
    }
    /**
     * Query [[Lock]]s by [[ChangeSet]] id that it was released with.
     * @param changesetId Id of the ChangeSet.
     * @returns This query.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if changeSetId is undefined or empty, or it contains an invalid [[ChangeSet]] id value.
     */
    byReleasedWithChangeSet(changeSetId) {
        Errors_1.ArgumentCheck.validChangeSetId("changeSetId", changeSetId);
        this.addFilter(`ReleasedWithChangeSet+eq+'${changeSetId}'`);
        return this;
    }
    /**
     * Query [[Lock]]s by [[ChangeSet]] index that it was released with.
     * @param changeSetIndex Index of the changeSet.
     * @returns This query.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley)
     * if changeSetIndex is undefined.
     */
    byReleasedWithChangeSetIndex(changeSetIndex) {
        Errors_1.ArgumentCheck.definedNumber("changeSetIndex", changeSetIndex);
        this.addFilter(`ReleasedWithChangeSetIndex+eq+${changeSetIndex}`);
        return this;
    }
    /**
     * Query [[Lock]]s by their instance ids.
     * @param locks Locks to query. They must have their BriefcaseId, LockType and ObjectId set.
     * @returns This query.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if locks array is undefined or empty, or it contains invalid [[Lock]] values.
     */
    byLocks(locks) {
        Errors_1.ArgumentCheck.nonEmptyArray("locks", locks);
        let filter = "$id+in+[";
        let index = 0;
        for (const lock of locks) {
            const id = getLockInstanceId(lock);
            Errors_1.ArgumentCheck.valid(`locks[${index}]`, id);
            if (0 !== index++)
                filter += ",";
            filter += `'${id}'`;
        }
        filter += "]";
        this.addFilter(filter);
        this._isMultiLockQuery = false;
        return this;
    }
    /**
     * Query unavailable [[Lock]]s. It will include all Locks owned by other [[Briefcase]]s and locks that were released with a newer [[ChangeSet]].
     * @param briefcaseId Id of the Briefcase.
     * @param lastChangeSetIndex Index of the last ChangeSet that user has pulled.
     * @returns This query.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if one of the values is undefined or briefcaseId is not in a valid [[Briefcase]] id value.
     */
    unavailableLocks(briefcaseId, lastChangeSetIndex) {
        Errors_1.ArgumentCheck.validBriefcaseId("briefcaseId", briefcaseId);
        Errors_1.ArgumentCheck.defined("lastChangeSetIndex", lastChangeSetIndex);
        let filter = `BriefcaseId+ne+${briefcaseId}`;
        filter += `+and+(LockLevel+gt+0+or+ReleasedWithChangeSetIndex+gt+${lastChangeSetIndex})`;
        this.addFilter(filter);
        return this;
    }
}
exports.LockQuery = LockQuery;
/**
 * Default page size which is used when querying Locks
 * @internal
 */
LockQuery.defaultPageSize = 10000;
/**
 * Handler for managing [[Lock]]s. Use [[IModelClient.Locks]] to get an instance of this class.
 * In most cases, you should use [ConcurrencyControl]($backend) methods instead. You can read more about concurrency control [here]($docs/learning/backend/concurrencycontrol).
 * @internal
 */
class LockHandler {
    /**
     * Constructor for LockHandler.
     * @param handler Handler for WSG requests.
     * @internal
     */
    constructor(handler) {
        this._handler = handler;
    }
    getRelativeUrl(iModelId, multilock = true, lockId) {
        return `/Repositories/iModel--${iModelId}/iModelScope/${multilock ? "MultiLock" : "Lock"}/${lockId || ""}`;
    }
    static convertLocksToMultiLocks(locks) {
        const map = new Map();
        for (const lock of locks) {
            const id = `${lock.lockType}-${lock.lockLevel}`;
            if (map.has(id)) {
                map.get(id).objectIds.push(lock.objectId);
            }
            else {
                const multiLock = new MultiLock();
                multiLock.changeState = "new";
                multiLock.briefcaseId = lock.briefcaseId;
                multiLock.seedFileId = lock.seedFileId;
                multiLock.releasedWithChangeSet = lock.releasedWithChangeSet;
                multiLock.releasedWithChangeSetIndex = lock.releasedWithChangeSetIndex;
                multiLock.lockLevel = lock.lockLevel;
                multiLock.lockType = lock.lockType;
                multiLock.objectIds = [lock.objectId];
                map.set(id, multiLock);
            }
        }
        return Array.from(map.values());
    }
    static convertMultiLocksToLocks(multiLocks) {
        const result = [];
        for (const multiLock of multiLocks) {
            for (const value of multiLock.objectIds) {
                const lock = new Lock();
                lock.objectId = value;
                lock.briefcaseId = multiLock.briefcaseId;
                if (lock.seedFileId)
                    lock.seedFileId = multiLock.seedFileId;
                lock.lockLevel = multiLock.lockLevel;
                lock.lockType = multiLock.lockType;
                result.push(lock);
            }
        }
        return result;
    }
    /** Augment update options with defaults returned by the DefaultLockUpdateOptionsProvider.
     * The options passed in by clients override any defaults where necessary.
     * @param options Options the caller wants to augment with the defaults.
     */
    async setupOptionDefaults(options) {
        if (!LockHandler._defaultUpdateOptionsProvider)
            LockHandler._defaultUpdateOptionsProvider = new DefaultLockUpdateOptionsProvider();
        return LockHandler._defaultUpdateOptionsProvider.assignOptions(options);
    }
    /** Send partial request for lock updates */
    async updateInternal(requestContext, iModelId, locks, updateOptions) {
        requestContext.enter();
        let requestOptions;
        if (updateOptions) {
            requestOptions = {};
            requestOptions.CustomOptions = {};
            if (updateOptions.deniedLocks === false) {
                requestOptions.CustomOptions.DetailedError_Codes = "false"; // eslint-disable-line @typescript-eslint/naming-convention
            }
            if (updateOptions.unlimitedReporting) {
                requestOptions.CustomOptions.DetailedError_MaximumInstances = "-1"; // eslint-disable-line @typescript-eslint/naming-convention
            }
            if (updateOptions.continueOnConflict) {
                requestOptions.CustomOptions.ConflictStrategy = "Continue";
            }
            if (Object.getOwnPropertyNames(requestOptions.CustomOptions).length === 0) {
                requestOptions = undefined;
            }
        }
        const result = await this._handler.postInstances(requestContext, MultiLock, `/Repositories/iModel--${iModelId}/$changeset`, LockHandler.convertLocksToMultiLocks(locks), requestOptions);
        requestContext.enter();
        return LockHandler.convertMultiLocksToLocks(result);
    }
    /** Update multiple [[Lock]]s. This call can simultaneously acquire new Locks and update states of already owned Locks. If large amount of Locks are updated, they are split across multiple requests. See [[LockUpdateOptions.locksPerRequest]]. Default is 2000 Locks per request.
     * @param requestContext The client request context.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param locks Locks to acquire. Requires briefcaseId, seedFileId to be set for every
     * Lock instance. They must be consistent throughout all of the Locks.
     * @param updateOptions Options for the update request. You can set this to change
     * how conflicts are handled or to handle different amount of Locks per request.
     * @returns Updated Lock values.
     * @throws [[ConflictingLocksError]] when [[LockUpdateOptions.deniedLocks]] is set and conflicts occurred. See [Handling Conflicts]($docs/learning/iModelHub/CodesAndLocksConflicts.md) for more information.
     * @throws [[AggregateResponseError]] when multiple requests where sent and more than 1 of the following errors occurred.
     * @throws [[IModelHubError]] with status indicating a conflict. See [Handling Conflicts]($docs/learning/iModelHub/CodesAndLocksConflicts.md) section for more information.
     * @throws [[IModelHubError]] with [IModelHubStatus.InvalidBriefcase]($bentley) when including locks with different briefcaseId values in the request.
     * @throws [[IModelHubError]] with [IModelHubStatus.OperationFailed]($bentley) when including multiple identical locks in the request.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async update(requestContext, iModelId, locks, updateOptions) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Requesting locks", () => ({ iModelId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        Errors_1.ArgumentCheck.nonEmptyArray("locks", locks);
        updateOptions = updateOptions || {};
        await this.setupOptionDefaults(updateOptions);
        const result = [];
        let conflictError;
        const aggregateError = new Errors_1.AggregateResponseError();
        for (let i = 0; i < locks.length; i += updateOptions.locksPerRequest) {
            const chunk = locks.slice(i, i + updateOptions.locksPerRequest);
            try {
                result.push(...await this.updateInternal(requestContext, iModelId, chunk, updateOptions));
                requestContext.enter();
            }
            catch (error) {
                requestContext.enter();
                if (error instanceof itwin_client_1.ResponseError) {
                    if (updateOptions && updateOptions.deniedLocks && error instanceof Errors_1.IModelHubError
                        && (error.errorNumber === bentleyjs_core_1.IModelHubStatus.LockOwnedByAnotherBriefcase || error.errorNumber === bentleyjs_core_1.IModelHubStatus.ConflictsAggregate)) {
                        if (conflictError) {
                            conflictError.addLocks(error);
                        }
                        else {
                            conflictError = ConflictingLocksError.fromError(error);
                        }
                        if (!updateOptions.continueOnConflict) {
                            throw conflictError;
                        }
                    }
                    else {
                        aggregateError.errors.push(error);
                    }
                }
            }
        }
        if (conflictError) {
            throw conflictError;
        }
        if (aggregateError.errors.length > 0) {
            throw aggregateError.errors.length > 1 ? aggregateError : aggregateError.errors[0];
        }
        bentleyjs_core_1.Logger.logTrace(loggerCategory, `Requested ${locks.length} locks for iModel`, () => ({ iModelId }));
        return result;
    }
    /** Get the [[Lock]]s that have been issued for the iModel.
     * @param requestContext The client request context.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param query Optional query object to filter the queried Locks or select different data from them.
     * @returns Resolves to an array of Locks matching the query.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async get(requestContext, iModelId, query = new LockQuery()) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Querying locks", () => ({ iModelId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        let locks;
        if (query.isMultiLockQuery) {
            const result = await this._handler.getInstances(requestContext, MultiLock, this.getRelativeUrl(iModelId), query.getQueryOptions());
            requestContext.enter();
            locks = LockHandler.convertMultiLocksToLocks(result);
        }
        else {
            locks = await this._handler.postQuery(requestContext, Lock, this.getRelativeUrl(iModelId, false), query.getQueryOptions());
            requestContext.enter();
            locks = locks.map((value) => {
                const result = new Lock();
                result.briefcaseId = value.briefcaseId;
                result.lockLevel = value.lockLevel;
                result.lockType = value.lockType;
                result.objectId = value.objectId;
                if (value.seedFileId)
                    result.seedFileId = value.seedFileId;
                return result;
            });
        }
        bentleyjs_core_1.Logger.logTrace(loggerCategory, `Queried ${locks.length} locks for iModel`, () => ({ iModelId }));
        return locks;
    }
    /** Delete all [[Lock]]s owned by the specified [[Briefcase]].
     * @param requestContext The client request context.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param briefcaseId Id of the Briefcase.
     * @throws [[IModelHubError]] with [IModelHubStatus.BriefcaseDoesNotExist]($bentley) if [[Briefcase]] with specified briefcaseId does not exist. This can happen if number was not given as a Briefcase id yet, or Briefcase with that id was already deleted.
     * @throws [[IModelHubError]] with [IModelHubStatus.UserDoesNotHavePermission]($bentley) if [[Briefcase]] belongs to another user and user sending the request does not have ManageResources permission.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async deleteAll(requestContext, iModelId, briefcaseId) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Deleting all locks from briefcase", () => ({ iModelId, briefcaseId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        Errors_1.ArgumentCheck.validBriefcaseId("briefcaseId", briefcaseId);
        await this.deleteAllLocksInChunks(requestContext, iModelId, briefcaseId);
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Deleted all locks from briefcase", () => ({ iModelId, briefcaseId }));
    }
    /** Helper method to iteratively delete chunks of [[Lock]]s for the specified [[Briefcase]] until there are no more left.
     * @param requestContext The client request context.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param briefcaseId Id of the Briefcase.
     */
    async deleteAllLocksInChunks(requestContext, iModelId, briefcaseId) {
        const relativeUrl = this.getRelativeUrl(iModelId, false, `DeleteChunk-${briefcaseId}`);
        const requestOptions = { timeout: { response: 45000 } };
        let shouldDeleteAnotherChunk = true;
        do {
            try {
                await this._handler.delete(requestContext, relativeUrl, requestOptions);
                requestContext.enter();
                bentleyjs_core_1.Logger.logTrace(loggerCategory, "Deleted a chunk of locks from briefcase", () => ({ iModelId, briefcaseId }));
            }
            catch (error) {
                requestContext.enter();
                if (error instanceof Errors_1.IModelHubError && error.errorNumber === bentleyjs_core_1.IModelHubStatus.LockChunkDoesNotExist)
                    shouldDeleteAnotherChunk = false;
                else
                    throw error;
            }
        } while (shouldDeleteAnotherChunk);
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Deleted all locks from briefcase", () => ({ iModelId, briefcaseId }));
    }
}
exports.LockHandler = LockHandler;
//# sourceMappingURL=Locks.js.map