"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module iModelHubClient
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ThumbnailHandler = exports.ThumbnailQuery = exports.LargeThumbnail = exports.SmallThumbnail = exports.Thumbnail = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const itwin_client_1 = require("@bentley/itwin-client");
const IModelHubClientLoggerCategories_1 = require("../IModelHubClientLoggerCategories");
const Errors_1 = require("./Errors");
const HubQuery_1 = require("./HubQuery");
const loggerCategory = IModelHubClientLoggerCategories_1.IModelHubClientLoggerCategory.IModelHub;
/** Base class for Thumbnails.
 * @public
 */
class Thumbnail extends itwin_client_1.WsgInstance {
}
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "instanceId")
], Thumbnail.prototype, "id", void 0);
exports.Thumbnail = Thumbnail;
/** Small [[Thumbnail]] class. Small Thumbnail is a 400x250 PNG image.
 * @public
 */
let SmallThumbnail = class SmallThumbnail extends Thumbnail {
};
SmallThumbnail = __decorate([
    itwin_client_1.ECJsonTypeMap.classToJson("wsg", "iModelScope.SmallThumbnail", { schemaPropertyName: "schemaName", classPropertyName: "className" })
], SmallThumbnail);
exports.SmallThumbnail = SmallThumbnail;
/** Large [[Thumbnail]] class. Large Thumbnail is a 800x500 PNG image.
 * @public
 */
let LargeThumbnail = class LargeThumbnail extends Thumbnail {
};
LargeThumbnail = __decorate([
    itwin_client_1.ECJsonTypeMap.classToJson("wsg", "iModelScope.LargeThumbnail", { schemaPropertyName: "schemaName", classPropertyName: "className" })
], LargeThumbnail);
exports.LargeThumbnail = LargeThumbnail;
/**
 * Query object for getting [[Thumbnail]]s. You can use this to modify the [[ThumbnailHandler.get]] results.
 * @public
 */
class ThumbnailQuery extends HubQuery_1.InstanceIdQuery {
    /**
     * Query [[Thumbnail]]s by [[Version]] id.
     * @param versionId Id of the version.
     * @returns This query.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if versionId is undefined or it is not a valid [GuidString]($bentley) value.
     * @internal @deprecated
     */
    byVersionId(versionId) {
        Errors_1.ArgumentCheck.validGuid("versionId", versionId);
        this.addFilter(`HasThumbnail-backward-Version.Id+eq+'${versionId}'`);
        return this;
    }
}
exports.ThumbnailQuery = ThumbnailQuery;
/**
 * Handler for retrieving [[Thumbnail]]s. Use [[IModelClient.Thumbnails]] to get an instance of this class.
 * @public
 */
class ThumbnailHandler {
    /**
     * Constructor for ThumbnailHandler.
     * @param handler Handler for WSG requests.
     * @internal
     */
    constructor(handler) {
        this._handler = handler;
    }
    /** Get relative url for tip Thumbnail requests.
     * @param contextId Id of the context ([[Project]] or [[Asset]]).
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param size Size of the thumbnail.
     */
    getRelativeContextUrl(contextId, iModelId, size) {
        return `/Repositories/Context--${this._handler.formatContextIdForUrl(contextId)}/ContextScope/${size}Thumbnail/${iModelId.toString()}/$file`;
    }
    /** Get relative url for Thumbnail requests.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param size Size of the thumbnail.
     * @param thumbnailId Id of the thumbnail.
     */
    getRelativeUrl(iModelId, size, thumbnailId) {
        return `/Repositories/iModel--${iModelId}/iModelScope/${size}Thumbnail/${thumbnailId || ""}`;
    }
    /** Check if given thumbnail is TipThumbnail.
     * @param thumbnail SmallThumbnail, LargeThumbnail or TipThumbnail.
     */
    isTipThumbnail(thumbnail) {
        return thumbnail.contextId !== undefined;
    }
    /** Download the thumbnail.
     * @param requestContext The client request context.
     *
     * @param url Url to download thumbnail.
     * @return String for the PNG image that includes the base64 encoded array of the image bytes
     */
    async downloadThumbnail(requestContext, url) {
        requestContext.enter();
        const options = {
            method: "GET",
            headers: { authorization: requestContext.accessToken.toTokenString() },
            responseType: "arraybuffer",
            agent: this._handler.getAgent(),
        };
        const response = await itwin_client_1.request(requestContext, url, options);
        requestContext.enter();
        const byteArray = new Uint8Array(response.body);
        if (!byteArray || byteArray.length === 0) {
            throw new Error("Expected an image to be returned from the query");
        }
        const base64Data = Base64.btoa(byteArray.reduce((acc, byte) => acc + String.fromCharCode(byte), ""));
        return `data:image/png;base64,${base64Data}`;
    }
    /** Download the latest iModel's thumbnail.
     * @param requestContext The client request context.
     * @param contextId Id of the iTwin context.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param size Size of the thumbnail. Pass 'Small' for 400x250 PNG image, and 'Large' for a 800x500 PNG image.
     * @return String for the PNG image that includes the base64 encoded array of the image bytes.
     */
    async downloadTipThumbnail(requestContext, contextId, iModelId, size) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, `Downloading tip ${size}Thumbnail`, () => ({ iModelId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("contextId", contextId);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        const url = await this._handler.getUrl(requestContext) + this.getRelativeContextUrl(contextId, iModelId, size);
        const pngImage = await this.downloadThumbnail(requestContext, url);
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, `Downloaded tip ${size}Thumbnail`, () => ({ iModelId }));
        return pngImage;
    }
    /** Get the [[Thumbnail]]s. Returned Thumbnails are ordered from the latest [[ChangeSet]] to the oldest.
     * @param requestContext The client request context.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param size Size of the thumbnail.
     * @param query Optional query object to filter the queried Thumbnails.
     * @return Array of Thumbnails of the specified size that match the query.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async get(requestContext, iModelId, size, query = new ThumbnailQuery()) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Querying iModel thumbnails", () => ({ iModelId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        let thumbnails = [];
        if (size === "Small")
            thumbnails = await this._handler.getInstances(requestContext, SmallThumbnail, this.getRelativeUrl(iModelId, size, query.getId()), query.getQueryOptions());
        else
            thumbnails = await this._handler.getInstances(requestContext, LargeThumbnail, this.getRelativeUrl(iModelId, size, query.getId()), query.getQueryOptions());
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Queried iModel thumbnails", () => ({ iModelId }));
        return thumbnails;
    }
    /** Download a [[Thumbnail]].
     * @param requestContext The client request context.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param thumbnail Small, Large or Tip thumbnail. Use [[ThumbnailHandler.get]] to get a [[SmallThumbnail]] or [[LargeThumbnail]] instance or provide Tip thumbnail information by constructing a [[TipThumbnail]] instance.
     * @return Base64 encoded string containing the PNG image.
     * @throws Error if a successful server response contains no content.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if one of the arguments is undefined or has an invalid value.
     * @throws [[ResponseError]] if a network issue occurs.
     */
    async download(requestContext, iModelId, thumbnail) {
        requestContext.enter();
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        if (this.isTipThumbnail(thumbnail)) {
            return this.downloadTipThumbnail(requestContext, thumbnail.contextId, iModelId, thumbnail.size);
        }
        const size = thumbnail instanceof SmallThumbnail ? "Small" : "Large";
        const thumbnailId = thumbnail.id;
        bentleyjs_core_1.Logger.logInfo(loggerCategory, `Downloading ${size}Thumbnail ${thumbnailId} for iModel`, () => ({ iModelId }));
        const url = `${await this._handler.getUrl(requestContext) + this.getRelativeUrl(iModelId, size, thumbnailId)}/$file`;
        const pngImage = await this.downloadThumbnail(requestContext, url);
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, `Downloaded ${size}Thumbnail ${thumbnailId} for iModel`, () => ({ iModelId }));
        return pngImage;
    }
}
exports.ThumbnailHandler = ThumbnailHandler;
//# sourceMappingURL=Thumbnails.js.map