"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module iModelHubClient
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserInfoHandler = exports.UserInfoQuery = exports.UserStatisticsHandler = exports.UserStatisticsQuery = exports.UserStatistics = exports.HubUserInfo = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const itwin_client_1 = require("@bentley/itwin-client");
const IModelHubClientLoggerCategories_1 = require("../IModelHubClientLoggerCategories");
const Errors_1 = require("./Errors");
const loggerCategory = IModelHubClientLoggerCategories_1.IModelHubClientLoggerCategory.IModelHub;
/** Information about the user, allowing to identify them based on their id.
 * @public
 */
let HubUserInfo = class HubUserInfo extends itwin_client_1.WsgInstance {
};
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Id")
], HubUserInfo.prototype, "id", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Name")
], HubUserInfo.prototype, "firstName", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Surname")
], HubUserInfo.prototype, "lastName", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Email")
], HubUserInfo.prototype, "email", void 0);
HubUserInfo = __decorate([
    itwin_client_1.ECJsonTypeMap.classToJson("wsg", "iModelScope.UserInfo", { schemaPropertyName: "schemaName", classPropertyName: "className" })
], HubUserInfo);
exports.HubUserInfo = HubUserInfo;
/** Statistics of user created and owned instances on the iModel.
 * @public
 */
let UserStatistics = class UserStatistics extends HubUserInfo {
};
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[HasStatistics].relatedInstance[Statistics].properties.BriefcasesCount")
], UserStatistics.prototype, "briefcasesCount", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[HasStatistics].relatedInstance[Statistics].properties.OwnedLocksCount")
], UserStatistics.prototype, "ownedLocksCount", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[HasStatistics].relatedInstance[Statistics].properties.PushedChangeSetsCount")
], UserStatistics.prototype, "pushedChangeSetsCount", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[HasStatistics].relatedInstance[Statistics].properties.LastChangeSetPushDate")
], UserStatistics.prototype, "lastChangeSetPushDate", void 0);
UserStatistics = __decorate([
    itwin_client_1.ECJsonTypeMap.classToJson("wsg", "iModelScope.UserInfo", { schemaPropertyName: "schemaName", classPropertyName: "className" })
], UserStatistics);
exports.UserStatistics = UserStatistics;
/**
 * Query object for getting User Statistics. You can use this to modify the [[UserStatisticsHandler.get]] results.
 * @public
 */
class UserStatisticsQuery extends itwin_client_1.WsgQuery {
    /**
     * Constructor for UserStatisticsQuery.
     */
    constructor() {
        super();
        this._statisticsPrefix = "HasStatistics-forward-Statistics";
        this._queriedByIds = false;
        this.select(`*`);
    }
    /**
     * Query single instance by its id.
     * @param id Id of the instance to query.
     * @returns This query.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) if id is undefined or empty.
     */
    byId(id) {
        Errors_1.ArgumentCheck.defined("id", id);
        this._byId = id;
        return this;
    }
    /**
     * Used by iModelHub handlers to get the id that is queried.
     * @internal
     */
    getId() {
        return this._byId;
    }
    /**
     * Query User Statistics by ids.
     * @param ids Ids of the users.
     * @returns This query.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if ids array is undefined or empty, or it contains undefined or empty values.
     */
    byIds(ids) {
        Errors_1.ArgumentCheck.nonEmptyArray("ids", ids);
        let filter = "$id+in+[";
        ids.forEach((id, index) => {
            Errors_1.ArgumentCheck.defined(`ids[${index}]`, id);
            if (index > 0)
                filter += ",";
            filter += `'${id}'`;
        });
        filter += "]";
        this.addFilter(filter);
        this._queriedByIds = true;
        return this;
    }
    /** Select all statistics. */
    selectAll() {
        return this.addSelect(`${this._statisticsPrefix}.*`);
    }
    /** Select currently owned [[Briefcase]]s count. */
    selectBriefcasesCount() {
        return this.addSelect(`${this._statisticsPrefix}.BriefcasesCount`);
    }
    /** Select total pushed [[ChangeSet]]s count. */
    selectPushedChangeSetsCount() {
        return this.addSelect(`${this._statisticsPrefix}.PushedChangeSetsCount`);
    }
    /** Select currently owned [[Lock]]s count. */
    selectOwnedLocksCount() {
        return this.addSelect(`${this._statisticsPrefix}.OwnedLocksCount`);
    }
    /** Select the last [[ChangeSet]] push date. */
    selectLastChangeSetPushDate() {
        return this.addSelect(`${this._statisticsPrefix}.LastChangeSetPushDate`);
    }
    /**
     * Returns whether was object queried by ids or no
     * @internal
     */
    get isQueriedByIds() {
        return this._queriedByIds;
    }
}
exports.UserStatisticsQuery = UserStatisticsQuery;
/**
 * Handler for querying [[UserStatistics]]. Use [[UserInfoHandler.Statistics]] to get an instance of this class.
 * @public
 */
class UserStatisticsHandler {
    /**
     * Constructor for UserStatistics. Should use @see IModelClient instead of directly constructing this.
     * @param handler Handler for WSG requests.
     * @internal
     */
    constructor(handler) {
        this._handler = handler;
    }
    /** Get relative url for UserStatistics requests.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param userId Id of the user.
     */
    getRelativeUrl(iModelId, userId) {
        return `/Repositories/iModel--${iModelId}/iModelScope/UserInfo/${userId ? userId : ""}`;
    }
    /** Get [[UserStatistics]].
     * @param requestContext The client request context.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param query Optional query object to filter the queried [[UserStatistics]] or select different data from them.
     * @returns Array of [[UserStatistics]] for users matching the query.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async get(requestContext, iModelId, query = new UserStatisticsQuery()) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Querying user statistics for iModel", () => ({ iModelId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        // if there are no specific selects defined, select all statistics
        if (query.getQueryOptions().$select === "*") {
            query.selectAll();
        }
        let userStatistics;
        if (query.isQueriedByIds) {
            userStatistics = await this._handler.postQuery(requestContext, UserStatistics, this.getRelativeUrl(iModelId), query.getQueryOptions());
        }
        else {
            userStatistics = await this._handler.getInstances(requestContext, UserStatistics, this.getRelativeUrl(iModelId, query.getId()), query.getQueryOptions());
        }
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, `Queried ${userStatistics.length} user statistics for iModel`, () => ({ iModelId }));
        return userStatistics;
    }
}
exports.UserStatisticsHandler = UserStatisticsHandler;
/** Query object for getting [[HubUserInfo]]. You can use this to modify the [[UserInfoHandler.get]] results.
 * @public
 */
class UserInfoQuery extends itwin_client_1.WsgQuery {
    constructor() {
        super(...arguments);
        this._queriedByIds = false;
    }
    /** Query UserInfo by user ids.
     * @param ids Ids of the users.
     * @returns This query.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if ids array is undefined or empty, or it contains undefined or empty values.
     */
    byIds(ids) {
        Errors_1.ArgumentCheck.nonEmptyArray("ids", ids);
        let filter = "$id+in+[";
        ids.forEach((id, index) => {
            Errors_1.ArgumentCheck.defined(`id[${index}]`, id);
            if (index > 0)
                filter += ",";
            filter += `'${id}'`;
        });
        filter += "]";
        this.addFilter(filter);
        this._queriedByIds = true;
        return this;
    }
    /** @internal */
    get isQueriedByIds() {
        return this._queriedByIds;
    }
    /** Query single instance by its id.
     * @param id Id of the instance to query.
     * @returns This query.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) if id is undefined or empty.
     */
    byId(id) {
        Errors_1.ArgumentCheck.defined("id", id);
        this._byId = id;
        return this;
    }
    /** Used by iModelHub handlers to get the id that is queried.
     * @internal
     */
    getId() {
        return this._byId;
    }
}
exports.UserInfoQuery = UserInfoQuery;
/** Handler for querying [[HubUserInfo]]. Use [[IModelClient.Users]] to get an instance of this class.
 * @public
 */
class UserInfoHandler {
    /** Constructor for UserInfoHandler.
     * @param handler Handler for WSG requests.
     * @internal
     */
    constructor(handler) {
        this._handler = handler;
    }
    /** Get the handler for querying [[UserStatistics]]. */
    get statistics() {
        return new UserStatisticsHandler(this._handler);
    }
    /** Get relative url for UserInfo requests.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param userId Id of the user.
     */
    getRelativeUrl(iModelId, userId) {
        return `/Repositories/iModel--${iModelId}/iModelScope/UserInfo/${userId || ""}`;
    }
    /** Get the information on users who have accessed the iModel.
     * @param requestContext The client request context.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param query Optional query object to filter the queried users or select different data from them.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async get(requestContext, iModelId, query = new UserInfoQuery()) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Querying users for iModel", () => ({ iModelId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        let users;
        if (query.isQueriedByIds) {
            users = await this._handler.postQuery(requestContext, HubUserInfo, this.getRelativeUrl(iModelId, query.getId()), query.getQueryOptions());
        }
        else {
            users = await this._handler.getInstances(requestContext, HubUserInfo, this.getRelativeUrl(iModelId, query.getId()), query.getQueryOptions());
        }
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Queried users for iModel", () => ({ iModelId }));
        return users;
    }
}
exports.UserInfoHandler = UserInfoHandler;
//# sourceMappingURL=Users.js.map