"use strict";
/*---------------------------------------------------------------------------------------------
* Copyright (c) Bentley Systems, Incorporated. All rights reserved.
* See LICENSE.md in the project root for license terms and full copyright notice.
*--------------------------------------------------------------------------------------------*/
/** @packageDocumentation
 * @module iModelHubClient
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.VersionHandler = exports.VersionQuery = exports.Version = void 0;
const bentleyjs_core_1 = require("@bentley/bentleyjs-core");
const itwin_client_1 = require("@bentley/itwin-client");
const IModelHubClientLoggerCategories_1 = require("../IModelHubClientLoggerCategories");
const Errors_1 = require("./Errors");
const HubQuery_1 = require("./HubQuery");
const loggerCategory = IModelHubClientLoggerCategories_1.IModelHubClientLoggerCategory.IModelHub;
/**
 * Named Version is a specific [[ChangeSet]] given a name to differentiate it from others. It can be used to represent some significant milestone for the iModel (e.g. a review version).
 * @public
 */
let Version = class Version extends itwin_client_1.WsgInstance {
};
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "instanceId")
], Version.prototype, "id", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Description")
], Version.prototype, "description", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Name")
], Version.prototype, "name", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.UserCreated")
], Version.prototype, "userCreated", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.CreatedDate")
], Version.prototype, "createdDate", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.ChangeSetId")
], Version.prototype, "changeSetId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "properties.Hidden")
], Version.prototype, "hidden", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[HasThumbnail].relatedInstance[SmallThumbnail].instanceId")
], Version.prototype, "smallThumbnailId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[HasThumbnail].relatedInstance[LargeThumbnail].instanceId")
], Version.prototype, "largeThumbnailId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[CreatedByApplication].relatedInstance[Application].properties.Id")
], Version.prototype, "applicationId", void 0);
__decorate([
    itwin_client_1.ECJsonTypeMap.propertyToJson("wsg", "relationshipInstances[CreatedByApplication].relatedInstance[Application].properties.Name")
], Version.prototype, "applicationName", void 0);
Version = __decorate([
    itwin_client_1.ECJsonTypeMap.classToJson("wsg", "iModelScope.Version", { schemaPropertyName: "schemaName", classPropertyName: "className" })
], Version);
exports.Version = Version;
/**
 * Query object for getting [[Version]]s. You can use this to modify the [[VersionHandler.get]] results.
 * @public
 */
class VersionQuery extends HubQuery_1.InstanceIdQuery {
    /**
     * Query [[Version]] by its name.
     * @param name Name of the Version.
     * @returns This query.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) if name is undefined or empty.
     */
    byName(name) {
        Errors_1.ArgumentCheck.defined("name", name);
        this.addFilter(`Name+eq+'${encodeURIComponent(name)}'`);
        return this;
    }
    /**
     * Query version by its [[ChangeSet]] id.
     * @param changesetId Id of the ChangeSet. Empty ChangeSet id can be provided to query iModel's baseline version.
     * @returns This query.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if changeSetId is undefined or not a valid [[ChangeSet.id]] format.
     */
    byChangeSet(changeSetId) {
        Errors_1.ArgumentCheck.validChangeSetId("changeSetId", changeSetId, true);
        this.addFilter(`ChangeSetId+eq+'${changeSetId}'`);
        return this;
    }
    /**
     * Query will additionally select ids of [[Thumbnail]]s for given [[ThumbnailSize]]s.
     * @returns This query.
     * @throws [[IModelHubClientError]] with [IModelHubStatus.UndefinedArgumentError]($bentley) or [IModelHubStatus.InvalidArgumentError]($bentley) if sizes array is undefined or empty.
     * @internal @deprecated
     */
    selectThumbnailId(...sizes) {
        Errors_1.ArgumentCheck.nonEmptyArray("sizes", sizes);
        if (!this._query.$select)
            this._query.$select = "*";
        for (const size of sizes) {
            this._query.$select += `,HasThumbnail-forward-${size}Thumbnail.*`;
        }
        return this;
    }
    /**
     * Query will additionally select data about application that created this [[Version]].
     * @returns This query.
     */
    selectApplicationData() {
        HubQuery_1.addSelectApplicationData(this._query);
        return this;
    }
    /**
     * Query only not hidden versions.
     * @returns This query.
     */
    notHidden() {
        this.addFilter("Hidden+eq+false");
        return this;
    }
}
exports.VersionQuery = VersionQuery;
/**
 * Handler for managing [[Version]]s. Use [[IModelClient.Versions]] to get an instance of this class.
 * @public
 */
class VersionHandler {
    /**
     * Constructor for VersionHandler.
     * @param handler Handler for WSG requests.
     * @internal
     */
    constructor(handler) {
        this._handler = handler;
    }
    /** Get relative url for Version requests.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param versionId Id of the version.
     */
    getRelativeUrl(iModelId, versionId) {
        return `/Repositories/iModel--${iModelId}/iModelScope/Version/${versionId || ""}`;
    }
    /** Get the named [[Version]]s of an iModel. Returned Versions are ordered from the latest [[ChangeSet]] to the oldest.
     * @param requestContext The client request context.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param query Optional query object to filter the queried Versions or select different data from them.
     * @returns Versions that match the query.
     * @throws [WsgError]($itwin-client) with [WSStatus.InstanceNotFound]($bentley) if [[InstanceIdQuery.byId]] is used and a [[Version]] with the specified id could not be found.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async get(requestContext, iModelId, query = new VersionQuery()) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Querying named versions for iModel", () => ({ iModelId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        const versions = await this._handler.getInstances(requestContext, Version, this.getRelativeUrl(iModelId, query.getId()), query.getQueryOptions());
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Queried named versions for iModel", () => ({ iModelId }));
        return versions;
    }
    /** Create a named [[Version]] of an iModel.
     * @param requestContext The client request context.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param changeSetId Id of the [[ChangeSet]] to create a named Version for. Empty ChangeSet id can be provided to create iModel's baseline version.
     * @param name Name of the new named Version.
     * @param description Description of the new named Version.
     * @returns Created Version instance.
     * @throws [[IModelHubError]] with [IModelHubStatus.UserDoesNotHavePermission]($bentley) if the user does not have ManageVersions permission.
     * @throws [[IModelHubError]] with [IModelHubStatus.ChangeSetDoesNotExist]($bentley) if the [[ChangeSet]] with specified changeSetId does not exist.
     * @throws [[IModelHubError]] with [IModelHubStatus.VersionAlreadyExists]($bentley) if a named [[Version]] already exists with the specified name.
     * @throws [[IModelHubError]] with [IModelHubStatus.ChangeSetAlreadyHasVersion]($bentley) if the [[ChangeSet]] with specified changeSetId already has a named [[Version]] associated with it.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async create(requestContext, iModelId, changeSetId, name, description) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Creating named version for iModel", () => ({ iModelId, changeSetId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        Errors_1.ArgumentCheck.validChangeSetId("changeSetId", changeSetId, true);
        Errors_1.ArgumentCheck.defined("name", name);
        let version = new Version();
        version.changeSetId = changeSetId;
        version.name = name;
        version.description = description;
        version = await this._handler.postInstance(requestContext, Version, this.getRelativeUrl(iModelId), version);
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Created named version for iModel", () => ({ iModelId, changeSetId }));
        return version;
    }
    /** Update the named [[Version]] of an iModel. Only the description can be changed when updating the named Version.
     * @param requestContext The client request context.
     * @param iModelId Id of the iModel. See [[HubIModel]].
     * @param version Named version to update.
     * @returns Updated Version instance from iModelHub.
     * @throws [[IModelHubError]] with [IModelHubStatus.UserDoesNotHavePermission]($bentley) if the user does not have ManageVersions permission.
     * @throws [[IModelHubError]] with [IModelHubStatus.VersionAlreadyExists]($bentley) if a named [[Version]] already exists with the specified name.
     * @throws [Common iModelHub errors]($docs/learning/iModelHub/CommonErrors)
     */
    async update(requestContext, iModelId, version) {
        requestContext.enter();
        bentleyjs_core_1.Logger.logInfo(loggerCategory, "Updating named version for iModel", () => ({ iModelId, changeSetId: version.changeSetId }));
        Errors_1.ArgumentCheck.defined("requestContext", requestContext);
        Errors_1.ArgumentCheck.validGuid("iModelId", iModelId);
        Errors_1.ArgumentCheck.validGuid("version.wsgId", version.wsgId);
        const updatedVersion = await this._handler.postInstance(requestContext, Version, this.getRelativeUrl(iModelId, version.id), version);
        requestContext.enter();
        bentleyjs_core_1.Logger.logTrace(loggerCategory, "Updated named version for iModel", () => ({ iModelId, changeSetId: version.changeSetId }));
        return updatedVersion;
    }
}
exports.VersionHandler = VersionHandler;
//# sourceMappingURL=Versions.js.map